package PureMessage::Manager::HTMLHelper;

use strict;

use Data::Dumper;
use ActiveState::Config qw(:path);;

sub new {
  my $CLASS = shift;
  my $this;
  $this->{OPTIONS} = undef;
  bless $this, $CLASS;
  return $this;
}

sub setDefaults {
  my $this = shift;
  foreach my $o (keys %{$this->{OPTIONS}} ) {
    $this->{OPTIONS}->{$o}->{value} = $this->{OPTIONS}->{$o}->{default} 
      unless defined $this->{OPTIONS}->{$o}->{value};
  }
}

# set either a single option or the entire hash if first param is a hash ref
sub set {
  my $this = shift;
  my $data = shift;
  if (ref($data) eq 'HASH') {
    $this->{OPTIONS} = $data;
  } else {
    $this->{OPTIONS}->{$data}->{value} = shift;
  }
}
  

sub found {
  my $this = shift;
  my $value = shift;
  if ( defined($this->{'OPTIONS'}->{$value}) ) {
    if ( $this->{'OPTIONS'}->{$value}->{type} eq 'text') {
      return $this->makeTextBox($value,$this->{OPTIONS}->{$value});
    } elsif ($this->{OPTIONS}->{$value}->{type} eq 'select') {
      return $this->makeSelectBox($value,$this->{OPTIONS}->{$value});
    } elsif ($this->{OPTIONS}->{$value}->{type} eq 'textarea') {
      return $this->makeTextArea($value,$this->{OPTIONS}->{$value});
    } elsif ($this->{OPTIONS}->{$value}->{type} eq 'checkbox') {
      return $this->makeCheckBox($value,$this->{OPTIONS}->{$value});
    } else {
      return "****** ERROR, not found $value\n";
    }
  } else {
    return "****** ERROR, not found $value\n";
  }
}


sub makeTextArea {
  my ($this,$name,$hash) = @_;
  my $text;
  $text = qq|$hash->{label} <textarea class=stretch NAME="$name" | 
        . $this->getHTMLOptions($hash) . qq|>$hash->{value}</textarea>\n|;

  $text .= "<span class=set_description>$hash->{description}</span>" 
    if exists $hash->{description};
  return $text;
}

sub makeCheckBox {
  my ($this,$name,$hash) = @_;
  my $text;
  $text = qq|$hash->{label} <input type=checkbox NAME="$name" | . $this->getHTMLOptions($hash);
  $text .= " checked " if $hash->{value} =~ /on|yes|checked|1/i;
  $text .= qq|>|;

  $text .= "<span class=set_description>$hash->{description}</span>" 
    if exists $hash->{description};
  return $text;
}

sub makeSelectBox {
  my ($this,$name,$hash) = @_;
  my $text;
  $text = qq|$hash->{label}<select class=stretch NAME="$name" | . $this->getHTMLOptions($hash) . qq|>\n|;
  foreach my $l (sort { $hash->{enum}->{$a} <=> $hash->{enum}->{$b} } keys %{$hash->{enum}}) {
    $text .= qq|<option value="$l" |;
    if (ref($hash->{'value'}) eq 'ARRAY') {
      # if it's an array of values then test each one
      foreach my $val (@{$hash->{'value'}}) {
        $text .= " SELECTED" if $val eq $l;
      }
      
    } else {
      $text .= " SELECTED" if $hash->{'value'} eq $l;
    }
    $text .= qq|>$hash->{enum}->{$l}\n|;
  }
  $text .= "</select>";
  $text .= "<span class=set_description>$hash->{description}</span>" 
    if exists $hash->{description};
  return $text;
}

sub getHTMLOptions {
  my $this = shift;
  my $hash = shift;
  my $text;
  return unless exists $hash->{options};
  if (ref($hash->{options}) eq 'HASH') {
    foreach my $options (keys %{$hash->{options}}) {
      if(defined $hash->{options}->{$options}) {
        $text .= qq| $options="$hash->{options}->{$options}"|;
      } else {
        $text .= qq| $options|;
      }
    }
  } else {
    # it must an array
     foreach my $options (@{$hash->{options}}) {
        $text .= qq| $options |;
     }
  }    

  return $text;
}

sub makeTextBox {
  my ($this,$name,$hash) = @_;
  my $text;
  $text .= qq|$hash->{label}<input class=stretch type=text VALUE="$hash->{value}" NAME="$name" | 
        . $this->getHTMLOptions($hash) . ">";
  $text .= "<span class=set_description>$hash->{description}</span>" 
    if exists $hash->{description};
  return $text;
}


sub filter {
  my ($this,$line_ref) = @_;
  $$line_ref =~ s/<TMPL_SP\s+NAME="?([^\s">]+)"?(ESCAPE=.*)?>/$this->found($1)/eig;
}

sub getMappedValue {
  my ($this,$key) = @_;
  my $OPTIONS = $this->{OPTIONS};
  if (exists $OPTIONS->{$key}->{emap}) {

    # then this has a mapping block
    my @values;
    if (ref($OPTIONS->{$key}->{value}) eq 'ARRAY') {
      @values = (@{$OPTIONS->{$key}->{value}});
    } else {
      @values = $OPTIONS->{$key}->{value};
    }
    my @return;
    foreach my $v (@values) {
      if (exists $OPTIONS->{$key}->{emap}->{$v}) {
        push(@return,$OPTIONS->{$key}->{emap}->{$v});
      }
    }
    return \@return if $#return > 0 or $#return == -1;
    return $return[0];
    ### not sure what to do, we can't figure out what value this is *supposed*
    ### to be, so just return the actual value I guess
  }

  return $OPTIONS->{$key}->{value}
}

sub getMap {
  my ($this,$key,$value) = @_;
  my $OPTIONS = $this->{OPTIONS};
  if (exists($OPTIONS->{$key}->{emap}) and exists($OPTIONS->{$key}->{enum}) ) {
    foreach my $ek (keys %{$OPTIONS->{$key}->{emap}}) {
      return $ek if $ek eq $value;
      return $ek if $OPTIONS->{$key}->{emap}->{$ek} eq $value;
    }
    # otherwise add it
    my $nk = $value;
    $OPTIONS->{$key}->{emap}->{"$nk"} = $value;
    $OPTIONS->{$key}->{enum}->{"$nk"} = $value;
    return $nk;
  }
}

sub getUnmappedValue {
  my ($this,$key,$value) = @_;
  my $OPTIONS = $this->{OPTIONS};
  if($OPTIONS->{$key}->{type} eq 'checkbox') {
    return $value; 
  } else {
    if (exists $OPTIONS->{$key}->{emap}) {
      return $this->getMap($key,$value);
    } else {
      return $value;
    }
  }
}



# parse the resulting hash and return a hash of key/value pairs that represent
# the data
sub getResult {
  my $this = shift;
  my $result;
  foreach my $key (keys %{$this->{OPTIONS}} ) {
    $result->{$key} = $this->getMappedValue($key);
  }

  return $result;
}

sub getValue {
  my ($this,$key) = @_;
  return $this->{OPTIONS}->{value} if exists $this->{OPTIONS}->{value};
  return;
}

# parse the resulting hash and return a hash of key/value pairs that represent
# the data
sub getResultPath {
  my $this = shift;
  my $fixups = shift;
  my $result = {}; # if it's not a ref bad things will happen when code is called first
  my $OPTIONS = $this->{OPTIONS};
  foreach my $key (keys %{$OPTIONS} ) {
    # if it's a save special block then the user will do it themselves
    next if exists($OPTIONS->{$key}->{save_special}) and $OPTIONS->{$key}->{save_special};
    
    if ($fixups and exists($fixups->{$key}) and ref($fixups->{$key}) eq 'CODE') {
      # this will set the value in the passed hash if it so desires
      &{$fixups->{$key}}($this,$key,$OPTIONS->{$key}->{path},$result)
          if exists $OPTIONS->{$key}->{path};
          
    } else {
      $result->{ $OPTIONS->{$key}->{path} } = $this->getMappedValue($key) 
        if exists $OPTIONS->{$key}->{path};
    }
  }
  return $result;
}

# given a result path as from the above sub it will set all values that match
# their listed default to __DEFAULT__ in that way we don't wack the value in 
# the conf file
sub resultPathDefault {
  my $this = shift;
  my $rp = shift;

  foreach my $key (keys %{ $this->{OPTIONS} }) {
    next unless exists $rp->{$this->{OPTIONS}->{$key}->{path}};
    $rp->{$this->{OPTIONS}->{$key}->{path} } = '__DEFAULT__'
      if $rp->{$this->{OPTIONS}->{$key}->{path} } eq $this->{OPTIONS}->{$key}->{default};
  }
  return $rp;
}


# helper routine to get a config file 
# May DIE!
sub loadConfig {
  my $this = shift;
  my $config_file = shift;
  my $hash;
  # this may die so eval!!
  
  my $obj = new ActiveState::Config(allow_includes => 0);
  $obj->parse_file($config_file);
  $hash = $obj->as_hash();
  $this->{OPTIONS} = $hash->{set};
  return $hash;
}

sub loadOptions {
  my $this = shift;
  $this->{OPTIONS} = shift;
}



sub import_data {
  my $this = shift;
  my $h = shift;
  my $fixes = shift;
  if (ref($h) =~ /^ActiveState::Config/) {
    $this->import_activestate_config($h,$fixes);
  } else {
    $this->import_hash($h,$fixes);
  }
}
sub import_activestate_config {
  my $this = shift;
  my $conf = shift;
  my $fixes = shift;
  my $root = $conf->root();
  my $OPTIONS = $this->{OPTIONS};
  foreach my $k (keys %$OPTIONS) {
    next unless exists $OPTIONS->{$k}->{path};
    my $n = path_node($root,$OPTIONS->{$k}->{path});
    next unless $n;
    if ($n and defined($n->value())) {
      # allow a config option to be set to unformatted, this will return the
      # canonized value, not the human readable value
      my $value = $OPTIONS->{$k}->{'canonize'} ? $n->value() : $n->value_formatted();
      if ($fixes and exists $fixes->{$k} and ref($fixes->{$k}) eq 'CODE') {
        $OPTIONS->{$k}->{value} = &{$fixes->{$k}}($this,$k,$value,$OPTIONS);
      } else {
        $OPTIONS->{$k}->{value} = $this->getUnmappedValue($k,$value);
      }
    } else {
      # then get our default
      $OPTIONS->{$k}->{value} = $OPTIONS->{$k}->{default};
    }
  }
  return $OPTIONS;
}

# expects an %in hash 
sub import_hash {
  my $this = shift;
  my $hash = shift;
  
  my $OPTIONS = $this->{OPTIONS};
  foreach my $set (keys %{$OPTIONS} ) {
    if ($OPTIONS->{$set}->{type} eq 'checkbox') {
      # then special case this, undef means no
      $OPTIONS->{$set}->{value} = exists $hash->{$set} ? $hash->{$set} : 'no';
    } elsif (exists $OPTIONS->{$set}->{emap}) {
      # then make sure that the value is in the map
      my @values;
      @values = ref($hash->{$set}) eq 'ARRAY' ? @{$hash->{$set}} : ($hash->{$set});
      foreach my $val (@values) {
        push(@{ $OPTIONS->{$set}->{value} },$this->getUnmappedValue($set,$val)) 
          if exists $hash->{$set};
      }
    
    } else {
      $OPTIONS->{$set}->{value} = $hash->{$set} if exists $hash->{$set};
    }
  }

  return $OPTIONS;
}




1;


