package PureMessage::Manager::HostDiscovery;
use strict;
use PureMessage::Version qw($VERSION $PREFIX);
use Exporter;
our @EXPORT_OK = qw( &discovery_probe_reply 
                     &discovery_probe_reply_decode
                     &discovery_probe_old_reply_decode
                   );

our @ISA = qw( Exporter );

# this package encapsulates the format of UDP discovery packets
# and supplies some helper routines for parsing thereof


our $MANAGER_DISCOVERY_MAGIC = 'PmMa'; # Magic to identify packets


sub discovery_probe_reply {
    my (%opt) = @_;
    # XXX this should croak if 'host' and 'port' are missing
    $opt{use_ssl} = 0 unless $opt{use_ssl}; # send a 0 for false instead of undef
    my $inst_defaults = -e "$PREFIX/etc/install-defaults.conf" || "";
    return "${MANAGER_DISCOVERY_MAGIC}:${VERSION}:"
           . "$opt{host}:$opt{port}:$opt{use_ssl}:$inst_defaults";
}

sub discovery_probe_reply_decode {
    my $response = shift;
    my ($magic,$ver,$host,$port,$use_ssl,$has_defaults) = split(/:/,$response);
    return unless $magic eq $MANAGER_DISCOVERY_MAGIC;
    return ( version => $ver, 
             host => $host, 
             port => $port, 
             use_ssl => $use_ssl,
             has_defaults => $has_defaults );
}

sub discovery_probe_old_reply_decode {
    my $response = shift;
    my ($host,$port,$use_ssl,@extra) = split(/:/,$response);
    # as a heuristic to recognize old format udp replies we will:
    # require that the port is defined and there are exactly 3 fields
    # present in the reply
    return unless $port and ! scalar(@extra);
    return ( version => 0, host => $host, port => $port, use_ssl => $use_ssl);
}

1;


