package PureMessage::Manager::Lang;
use Exporter;
use strict;
use PureMessage::Version qw ($PREFIX);
use ActiveState::Config::Simple qw(config_parse_file);
use PureMessage::Manager::Config qw(%web_config);
use PureMessage::Lang qw(set_lang_dir);
our @ISA = qw(Exporter);
our @EXPORT = qw( %text &text &mtext &ntext &lang_text_utf8 &lang_ntext_utf8);

# reads the lang file and exports an appropriate %text hash
# TODO XXX this needs support for actually determining the language at runtime.
# Also do we really need support for global lang files? -- Yes, we do...

our $language = 'en';
our $lang_dir = 'etc/manager/lang/installed';

my $file = $web_config{lang_file} || "$PREFIX/etc/manager/$language.lang";
my $namespace = $ENV{REQUEST_URI} || "";
$namespace =~ s/([^\/]+)$//;

our %text;

my %lang_text;
my $LANG_PREFIX = "$PREFIX/etc/manager";

if ( -f $file ) {
    *text = config_parse_file($file) || {};

    # Comment this when all $text{} are replaced with text()
    init(\%text);
}
else {
    warn "Can't open translation file $file";
}

set_lang_dir($lang_dir);

# this flattens all uniq keys
sub init {
    my $node = shift;
    while ( my ($k, $v) = each %$node ) {
        if ( ref($v) eq "HASH" ) {
            init($v);
        }
        else {
            $text{$k} = $v unless exists $text{$k};
        }
    }
}

sub set_lang {
    my $lang = shift;
    my %opt = @_;
    my $new_text;
    my $new_lang_file = "$LANG_PREFIX/$lang.lang";
    if ( -f $new_lang_file ) {
	$new_text = config_parse_file($new_lang_file, undef, %opt) || {};
       	# Comment this when all $text{} are replaced with text()
	init($new_text);
    } else {
	warn "Can't open translation file $new_lang_file";
    }
    $lang_text{$lang} = $new_text;
}

sub set_namespace {
    $namespace = shift;
}

sub get_namespace {
    return $namespace;
}
# same as text but takes a namespace name for text enties
# ntext($namespace,$message [@substitutions])
sub ntext {
  my $old = $namespace;
  $namespace = shift;
  my $result = text(@_);
  $namespace = $old; 
  return $result;
}

# same as text but uses a callers package name for namespace for text enties
# mtext($message [@substitutions])
sub mtext {
  return ntext(scalar(caller),@_);
}

# text(message, [substitute]+)
sub text {
    my $key_lang = shift;
    my $key;
    my $get_text;
    # check to see if it is getting the language specific text
    if (ref($key_lang) eq 'HASH') {
	$key = $key_lang->{'key'};
	if (ref($key_lang->{'get_text'}) eq 'HASH') {
	    $get_text = $key_lang->{'get_text'};
	}
    } else {
	$key = $key_lang;
	$get_text = \%text;
    }
    my @ns = split(/[\.\/]/, $namespace);
    my $now;
    my $found;
    foreach ( @ns ) {
        next unless $_;
        $now = $now ? $now->{$_} : $get_text->{$_};
        next unless ref($now);
        last unless $now;
        if (ref($now) eq "ARRAY") {
          my %new_now;
          foreach my $row (@$now) {
            
            foreach my $key (keys %$row) {
                if(exists($new_now{$key})) {
                  $new_now{$key}->{$_} = $row->{$key}->{$_} foreach keys(%{$row->{$key}});
                } else { 
                  $new_now{$key} = $row->{$key};
                }
            }
          }
          $now = \%new_now;
        }
        $found = $now->{$key} if $now->{$key} and ref($now->{$key}) ne "HASH";
    }
    $found = $get_text->{$key} unless $found or ref($get_text->{$key}) eq "HASH";
    if ( ref($found) eq "ARRAY" ) {
        $found = join("\n", @$found);
    }
    return interpolate($found, @_) if $found;
    return $key;
}

# XXX lang_text_utf8() and lang_ntext_utf8() always returns their
# strings in utf-8. All the other text methods in this library return
# the text in iso-8859-1. This Lang lib should be refactored or replaced
# so that this is made more obvious.
sub lang_text_utf8 {
    my $lang = shift;
    my $key = shift;

    # if the language doesn't already exist in hash, add it
    if ((! exists($lang_text{$lang})) || (! defined($lang_text{$lang}))) {
	set_lang($lang, text_encoding => 'utf8');
    }
    text({ 'key' => $key, 'get_text' => $lang_text{$lang} }, @_);
}

# same as lang_text_utf8 but takes a namespace name for text enties
# lang_ntext_utf8($namespace,$language,$message [@substitutions])
sub lang_ntext_utf8 {
  my $old = $namespace;
  $namespace = shift;
  my $result = lang_text_utf8(@_);
  $namespace = $old; 
  return $result;
}


sub interpolate {
    my $rv = shift;
    my $i;
    for($i = 1; $i <= @_; $i++) {
        $rv =~ s/\$$i/$_[$i-1]/g;
    }
    return $rv;
}

1;
