package PureMessage::Manager::RPCRequest;

use strict;
use warnings;
use Exporter;
our @ISA = qw(Exporter);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %self = ();
    return bless( \%self, $class);
}

sub call {
    my $self = shift;
    my $class = shift || 'PureMessage::Manager::RPCResponse';
    my $method = shift || 'doc';
    $class = "PureMessage::Manager::RPCResponse::$class" if $class !~ /::/;
    my @data = @_;
    my $r;
    eval "require $class";
    if ( $@ ) {
        $r = PureMessage::Manager::RPCResponse->new();
        $r->raise_error("Service not found.");
    }
    else {
        $r = $class->new();
        $r->{REQUEST} = $self;
        if ( $r->can($method) ) {
            eval { $r->$method(@data) };
            if ( $@ ) {
               $r->raise_error("RPC call failed: $!");
            }
            else {
               if ( ! $r->{done_headers} ) { 
                   # no data sent by the method
                   $r->dispatch({'UNDEF'=>1})
               }
               $r->serve("\cD"); # EOF
            }
        }
        else {
            $r->raise_error("Method $method is not implemented.");
        }
    }
    return $r;
}

1;

# provides data serialization functionality
package PureMessage::Manager::RPCResponse;
use Text::CSV_XS;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %self = ();
    $self{headers} = { 'Content-Type' => 'text/csv' };
    $self{serializer} = Text::CSV_XS->new({binary=>1});
    return bless( \%self, $class);
}

sub raise_error {
    my $self = shift;
    my $err_msg = shift;
    $self->dispatch( { ERROR => $err_msg });
}

sub dispatch_headers {
    my $self = shift;
    my $sample = shift;
    while ( my ($k, $v) = each %{$self->{headers}} ) {
        $self->serve("$k: $v\r\n");
    }
    
    # end of HTTP headers
    $self->serve("\r\n");
    
    # print column names if needed
    if ( ! $self->{meta_data} and $sample ) {
        if ( ref($sample) eq "HASH" ) {
            $self->{meta_data} = [ keys %{$sample} ];
        }
        else {
            $self->{meta_data} = ['DATA'];
        }
    }
    $self->serve($self->serialize(@{$self->{meta_data}})) if $self->{meta_data};
    $self->{done_headers} = 1;
}

sub dispatch_raw {
    # dispatch preserialized data
    my $self = shift;
    if ( ! $self->{done_headers} ) {
        $self->dispatch_headers();
    }
    foreach ( @_ ) {
        $self->serve($_);
    }
}

sub dispatch {
    my $self = shift;
    my @data = @_;
    if ( ! $self->{done_headers} ) {
        $self->dispatch_headers($data[0]);
    }
    foreach my $record ( @data ) {
        if ( ref($record) eq "HASH" ) {
            $self->serve( $self->serialize( map {$record->{$_}} @{$self->{meta_data}} ) );
        }
        elsif ( ref($record) eq "ARRAY" ) {
            $self->serve($self->serialize(@{$record}));
        }
        else {
            $self->serve($self->serialize($record));
        }
    }
}

sub serialize {
    my $self = shift;
    my @data = @_;
    if ( $self->{serializer}->combine(@data) ) {
        return $self->{serializer}->string()."\n";
    }
    else {
	# XXX should do better handling for "run-time" errors
        die "ERROR\nData serialization error";
    }
}

sub serve {
    my $self = shift;
    print $_ foreach @_;
}

# default method
sub doc {
    my $self = shift;
    $self->dispatch("PureMessage Manager RPC Framework");
}

1;
