package PureMessage::Manager::Service;
use strict;
use warnings;
use Carp;
use PureMessage::Manager::Setup qw(manager_connect_test);
use PureMessage::Manager::Config qw(%web_config);
use ActiveState::Handy qw(run);
use ActiveState::Unix::ProcInfo qw(proc_info);

use base 'Exporter';
our @EXPORT_OK = qw(
	manager_pid
	manager_is_running
	manager_start
	manager_stop
	manager_restart
	manager_status
	manager_print_status
);

sub manager_pid{
	croak("Configuration error: $PureMessage::Manager::Config::conf_error")
	    if $PureMessage::Manager::Config::conf_error;
	    
	my $pid_file = $web_config{'pidfile'};
	my $pid = 0;
	return $pid unless -e $pid_file;
	if(open(PF, "<$pid_file")){
		$pid = <PF>;
		chomp($pid);
		close PF;
	}
	return $pid;
}

sub manager_is_running{
	my $pid = manager_pid();
	return 0 unless $pid;
	my $info = proc_info ('root_pid' => $pid);
	return 0 unless $info && $$info{$pid}; 
	return 1;
}

sub manager_start{
	return 1 if manager_is_running();
	croak("Configuration error: $PureMessage::Manager::Config::conf_error")
	    if $PureMessage::Manager::Config::conf_error;
    
	my $cmd = "$web_config{'root'}/miniserv.pl";
	croak("Web server program does not have permissions to run.")
		unless -x $cmd;
	run("\@$cmd");
	return 1;
}

sub manager_stop{
	return 1 unless manager_is_running();
	my $pid = manager_pid();
	
	if ( kill(0, $pid) ) {
	    
            return unless kill(15, $pid); # send a TERM signal
            
            my $count = 0;

            # try for 10 seconds for the manager to die
            while ( $count < 10 and manager_is_running() ) {
                sleep(1);
                $count++;
            }
            
            return manager_is_running() ? 0 : 1;
     
	}
        
	return 0;
}

sub manager_restart{
	my $c = 1;
        # Don't allow ourselves to be killed. If we are running under
        # miniserv.pl we will die naturally after we are done here 
        # since cgi's are never piplelined
        local $SIG{TERM} = 'IGNORE'; 
	if(manager_is_running()){
		$c = manager_stop();
	}
	return 0 unless $c;
	return manager_start();
}

sub manager_status{
	my %info = ();
	$info{'port'}     = $web_config{'port'};
	$info{'pid'}      = manager_pid();
	$info{'running'}  = manager_is_running();
	$info{'https'}    = $web_config{'ssl'};
	$info{'hostname'} = PureMessage::_sys::gethostname();
	my $https         = $web_config{'ssl'} ? 'https' : 'http';	
	$info{'url'}      = "$https://$info{'hostname'}:$info{'port'}";
	return \%info;
}

sub manager_print_status{
	my $info = manager_status();
	unless ($$info{'running'}) {
		print "Manager is not running.\n";
		return 0;
	}
	print "Manager:\n";
	print "  pid:     $$info{'pid'}\n";
	print "  url:     $$info{'url'}\n";
	return 1;
}

1;
