package PureMessage::Manager::Utils;
use strict;
use warnings;
use Exporter;
our @ISA = qw(Exporter);
use URI::Escape qw(uri_escape uri_unescape);

my @html_functions = qw(
    html_escape
    urlize
    un_urlize
);

our @EXPORT_OK = (
    @html_functions,
    'check_ipaddress'
);

our %EXPORT_TAGS = (
    'html' => \@html_functions,
    'all'  => \@EXPORT_OK,
);

# html_escape
# Convert &, < and > codes in text to HTML entities
# from web-lib.pl
sub html_escape
{
    my $tmp = shift;
    return unless defined($tmp);
    $tmp =~ s/&/&amp;/g;
    $tmp =~ s/</&lt;/g;
    $tmp =~ s/>/&gt;/g;
    $tmp =~ s/\"/&quot;/g;
    $tmp =~ s/\'/&#39;/g;
    return $tmp;
}

# check_ipaddress(ip)
# Check if some IP address is properly formatted
# from web-lib.pl
sub check_ipaddress
{
    return $_[0] =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ &&
	$1 >= 0 && $1 <= 255 &&
	$2 >= 0 && $2 <= 255 &&
	$3 >= 0 && $3 <= 255 &&
	$4 >= 0 && $4 <= 255;
}

# encode a string to 'application/x-www-form-urlencoded' format
# http://www.w3.org/MarkUp/html-spec/html-spec_8.html#SEC8.2.1
sub urlize {
    my $rv = uri_escape($_[0]);
    $rv =~ s/\%20/+/g;
    return $rv;
}

# decode a string from 'application/x-www-form-urlencoded' format
# both '%20' and '+' are legal encodings of ' ', so we need to 
# get rid of '+', which uri_unescape won't deal with.
# Note: a literal '+' will have been encoded as '%2B'.
sub un_urlize {
    (my $rv = $_[0]) =~ tr/+/ /;
    return uri_unescape($rv);
}

1;
