package PureMessage::Manager::Widget;
use strict;
use ActiveState::Config;
use Exporter;
use Data::Dumper;
use HTML::Template;
use vars qw(@EXPORT @ISA);
use PureMessage::Manager::Config qw(%web_config);
use PureMessage::Manager::Utils qw( html_escape);

@EXPORT = qw( icon template button buttons dump_hash);
@ISA = qw(Exporter);

# a singleton
our $WIDGET_CONFIG = { icons => "$web_config{root}/img/",
                       icon_base => "/img/",
                       templates => $web_config{"tmpl_path"},
                     };

my $WIDGET_FORM_NAME = '__W!';

sub new {
    my $proto = shift;
    my $CLASS = ref($proto) || $proto;
    my %options = @_;
    my $this = { @_ };
    bless $this, $CLASS;
    $this->_init();
    return $this;
}

# our little class factory
sub widget {
        my $this = shift;
        my %options = @_;
        my $class = $options{CLASS};
        return unless $class;
        delete($options{CLASS});

        eval "require $class";
        my $object = $class->new(%options);
        return $object;
}


sub errors {
        my $this = shift;
        if (@_) {
                push(@{$this->{errors}},@_)
        } else {
               # avoid warnings
               
               if (ref($this->{errors}) eq 'ARRAY') {
                 return @{$this->{errors}};
               } else {
                 return;
               }
        }
}

sub get {
        my $this = shift;
        my $key = shift;
        # look for the value in the user supplied options
        # then the widget specific options
        # then the global widget options
        # 
        if ( defined($this->{$key}) ) {
                return $this->{$key};
        } elsif ( defined( $this->{CONFIG}->{widget}->{ref($this)}->{$key} ) ) {
                return $this->{CONFIG}->{widget}->{ref($this)}->{$key};
        } else {
                return $this->{CONFIG}->{$key};
        }
}

sub set {
        my $this = shift;
        my $key = shift;
        $this->{$key} = shift;
}

sub get_text {
        my $this = shift;
        my $data = shift;
        my %options = @_;
        my $result;
        
        if(UNIVERSAL::isa($data, "PureMessage::Manager::Widget")) {
                $result .= $data->output();
        } elsif(ref($data) eq 'ARRAY') {
        
                foreach my $row (@$data) {
                        $result .= $this->get_text($row);
                        $result .= $this->get_text($options{'post_row'}) if $options{'post_row'}
                }
                
        } elsif(ref($data) eq 'HASH') {
                foreach my $key (keys %$data) {
                        $result .= $this->get_text($data->{$key});
                        $result .= $this->get_text($options{'post_row'}) if $options{'post_row'}
                }
        } elsif(ref($data) eq 'CODE') {
                if($options{'method'}) {
                        $data .= &$data($this,%{$options{options}});
                } else {
                        $data .= &$data(%{$options{options}});
                }
        } else {
                $result .= $data;
        }

        return $result;

}

sub section_header {
  my $this = shift;
  my $title = shift;
  return $this->widget( CLASS => 'PureMessage::Manager::Widget::ThemeSectionHeader',
                        title => $title
                      )->output();
                     
}

sub make_form_name {
        my $this = shift;
        my $name = shift;
        if (defined($this->{name})) {
                # then make a name using the name value
                return "$WIDGET_FORM_NAME!$this->{name}!$name";
        } else {
                # else make one using our class name
                my $class = ref($this);
                return "$WIDGET_FORM_NAME!$class!$name";
        }
}

sub is_form_name {
        my $this = shift;
        my $name = shift;
        if ($name =~ /^$WIDGET_FORM_NAME!([^!]+)!([^!]+)$/) {
                # it is one, return the widget name and base name
                return ($1,$2);
        } else {
                # it's not one
                return;
        }
}

sub config_default {
  my $this = shift;
  my %info = @_;
  foreach my $key (%info) {
    $this->{CONFIG}->{widget}->{ref($this)}->{$key} = $info{$key};
  }
}

sub read_input {
        my $this = shift;
        $this->{REQUEST} = shift; # just in case we need it later
        my $only_for_widget = shift;
        my %widgets = ();
        my %data = ();
        my $found = 0;
        return unless $this->{REQUEST};
        foreach my $key ( $this->{REQUEST}->query_names()) {
                my @names = $this->is_form_name($key);
                if ($#names >= 0) {
                        # skip this input if it's not for the specific named
                        # widget
                        next if $only_for_widget and $names[0] ne $only_for_widget;
                        $widgets{$names[0]} = 1;
                        $data{$names[1]} = $this->{REQUEST}->query($key);
                        $found = 1;
                } 
        }
        $this->{INPUT} = \%data if $found;
        return keys(%widgets);
        
}

sub notification_header_str {
        my $this = shift;
        return <<EOF;
<table class="light" width="100%" cellspacing="0" cellpadding="4" border="0">
<tr>
<td class="tableRowAlt">
EOF
}

sub notification_footer_str {
        my $this = shift;
        return <<EOF;
        </td>
        </tr>
</table>       
EOF
}


sub result_data {
        my $this = shift;
        return $this->{INPUT};
}

sub value_is_true {
        my $this = shift;
        my $value = shift;
        return 1 if $value =~ /^yes|true|checked|ok|confirm|confirmed$/i;
        return;
}

sub query {
        my $this = shift;
        my $param = shift;
        if ($param) {
                return $this->{INPUT}->{$param};
        } else {
                return keys %{$this->{INPUT}};
        }
}

sub result_is_true {
        my $this = shift;
        return 1 if $this->value_is_true($this->query('RESULT')); 
        return;
}

# prime our semi-static stuff
sub _init {
        my $this = shift;
        $this->{CONFIG}->{templates} = $web_config{"tmpl_path"};
        $this->{CONFIG}->{icons} = "$web_config{root}/img/";
        $this->{CONFIG}->{icon_base} = "/img/";
}


sub debug {
  my $this = shift;
  $this->{debug} = shift;
}

# DPL this is probably a bad idea. A superclass should not depend on a
# subclass. :(
sub output_debug {
  my $this = shift;
  my $name = $this->get('name');
  my $title = ref($this) . ": $name";
  my $pane = $this->widget(CLASS => 'PureMessage::Manager::Widget::PaneBox',
                     content => '<PRE>' . html_escape(Dumper($this)) . '</PRE>',
                     title => "$title",
                     width => 300,
                    );

  return $pane->output();
                            
}
sub _debug_hash {
  my $this = shift;
  my ($title,$hash) = @_;
  return dump_hash($title,$hash);
}

sub dump_hash {
    my ($title,$hash) = @_;
    my $w = new PureMessage::Manager::Widget;
    my $txt_IN = '';
    TOP: foreach my $key (keys %$hash) {
            if (ref($hash->{$key}) eq 'ARRAY') {
                $txt_IN .= "<LI> " . html_escape($key) . "\n <UL> ";
                foreach my $element (@{$hash->{$key}}) {
                    if (ref($element) eq 'ARRAY' 
                          or ref($element) eq 'HASH') {
                        # bail out if too deep
                        $txt_IN = "<PRE> ";
                        $txt_IN .= html_escape(Dumper($hash));
                        $txt_IN .= "</PRE>";
                        last TOP;
                    }
                    $txt_IN .= "<LI>" .  html_escape($element). "\n";
                }
                $txt_IN .= "</UL>\n";
            }
            elsif(ref($hash->{$key}) eq 'HASH') {
                $txt_IN .= "<LI> " . html_escape($key) . "\n <UL> ";
                foreach my $element (keys %{$hash->{$key}}) {
                    if (ref($hash->{$key}->{$element}) eq 'ARRAY' 
                        or ref($hash->{$key}->{$element}) eq 'HASH') {
                        # bail out if too deep
                        $txt_IN = "<PRE> ";
                        $txt_IN .= html_escape(Dumper($hash));
                        $txt_IN .= "</PRE>";
                        last TOP;
                    }
                    $txt_IN .= "<LI> " . html_escape($element) . " = " . html_escape($hash->{$key}->{$element}) . "</LI>\n";
                }
                $txt_IN .= "</UL>\n";
            }
            elsif(ref($hash->{$key})) {
                # bail out we don't know what to do
                #
                $txt_IN = "<PRE>";
                $txt_IN .= html_escape(Dumper($hash));
                $txt_IN .= "</PRE>";
                last TOP;
            } 
            else {
                $txt_IN .= "<LI> $key = " . html_escape(defined($hash->{$key}) ? $hash->{$key} : '') . "\n";
            }
        }
  
        $txt_IN = "<UL> $txt_IN </UL>\n";
  
    my $pane = $w->widget(CLASS => 'PureMessage::Manager::Widget::PaneBox',
                     content =>  $txt_IN,
                     title => $title,
                     width => 300,
                    );
    return  $pane->output();
}



#
# functions below.
# 


sub icon {
        my $name = shift;
        return unless $name; # catch uninitialized warnings
        # if no icon extension assume it's a gif file
        $name .= '.gif' unless $name =~ /\w\.\w/;
        if (-f "/img/$name") {
                # return a relative ref to this modules icon directory
                return "/img/$name";
        } elsif( -f "img/$name") {
                # return a relative ref to this modules image directory
                return "img/$name";
        } elsif( -f "$WIDGET_CONFIG->{icons}/$name") {
                return "$WIDGET_CONFIG->{icon_base}$name";
        } elsif ( not -f "$name" ) {
            return "/img/x.gif";
        }
        return $name; # else return what we were given  
}

sub template {
        my $name = shift;
        my @options = @_ if @_;
        push(@options,die_on_bad_params => 0);
        # if no template extension assume it's html 
        $name .= '.html' unless $name =~ /\w\.\w/;
        my $t = HTML::Template->new(
                    filename => "$name",
                    path     => $web_config{'tmpl_path'},
                    die_on_bad_params => 0,
                    loop_context_vars => 1,
                    @options);
        return $t; # else return nothing  
}


sub button {
        my $bd = shift;
        my %options = @_;
        my $data;
        # image, text, custom
        if ($bd->{type} eq 'image' or $bd->{type} eq 'icon') {
                # then we should find an icon link
                if($bd->{name}) {
                        $data = _form_input_img($bd,%options);
                } elsif($bd->{href}) {
                        $data = _link_img($bd,%options);
                } else {
                        $data = _image($bd,%options);
                }
                
        } elsif($bd->{type} eq 'text') {
                if ($bd->{name} and !$bd->{text}) {
                        $data = _form_input_button($bd,%options);
                }elsif($bd->{name} and $bd->{text}) {
                        $data = _form_text_input($bd,%options);
                }elsif($bd->{href}) {
                        $data = _link_text($bd,%options);
                } else {
                        $data = defined($bd->{text}) ? $bd->{text} : ''; # catch uninitialized
                }
                
        } elsif($bd->{type} eq 'hidden') {
            my $id = $bd->{id} ? " id='$bd->{id}' " : '';
            $data = "<input type=hidden name='$bd->{name}' value='$bd->{value}'$id>\n";
        }else {
                # type is custom or unknown
                $data = _format_custom($bd,%options);
        }
        
        return $data;

}

sub _format_custom {
        my %bd = %{shift(@_)};
        my %options = @_;
        my $data;
        $data = $bd{text};
        my $class = " class=\"$options{class}\"";
        if ($bd{wrap}) {
                $data = <<EOF;
    <table border="0" cellpadding="3" cellspacing="0"  width='100%'>
	<tr><td $class valign="middle" nowrap="nowrap">$data</td>
	</tr>
    </table>
EOF
        }

        return $data;

}

# formats the passed hash as a hidden button for saving into a toolbar
sub state_buttons {
        my $this = shift;
        my $ref = shift;
        my @data;
        my $button;
        foreach my $key (keys %{$ref}) {
                # if it has an array value make multiple hidden fields, one for
                # each value
                if (ref($ref->{$key}) eq 'ARRAY') {
                        foreach my $v (@{$ref->{$key}}) {
                                push(@data, { type=>'hidden', name=>$key, value=>$v });
                        }
                } else {
                        push(@data, { type=>'hidden', name=>$key, value=>$ref->{$key} });
                }
        }
        return @data;
        
}

sub format_state {
  my $this =shift;
  my $state = $this->get('state');
  if(ref($state) eq 'HASH') {
    my @buttons;
    foreach my $k (keys %$state) {
      push(@buttons,{ type=>'hidden', name=>$k, value=>$state->{$k}});
    }
    return join('',buttons(\@buttons));
  }
}


sub buttons {
        my ($bd,%options) = @_;
        my @data;

        foreach my $button (@$bd) {
                push(@data,button($button,%options));
        }
        return @data;
}

# generate an option list with the current value selected
# $data_hash has the format { form_element_name => 'text description' ..}
sub option_list {
        my $this = shift;
        my ($name,$data_hash,$selected,$extra,$order) = @_;
        $extra = '' unless $extra;
        my $return;
        my @order = @$order if $order;
        @order  = sort keys %$data_hash unless $order;
        $return = "<select name='$name' $extra>\n";
        foreach my $option (@order) {
                my $SELECTED = $option eq $selected ? 'SELECTED' : '';
                $return .= "<option value='$option'$SELECTED>$data_hash->{$option}\n";
        }
        $return .= "</select>\n";
}

sub _button_format {
    my ($left, $right, $class, $style) = @_;
    $class = defined($class) ? " class=\"$class\"" : '';
    $left = defined($left) ? $left : '';
    $right = defined($right) ? $right : '';
    if (defined($style) and $style eq "toolbar" ) {
        return <<EOT;
        <table border="0" cellpadding="3" cellspacing="0">
	<tr><td $class valign="middle">$left</td>
            <td $class valign="middle" nowrap="nowrap">$right</td>
            <td $class><img src="/img/x.gif" width="1" height="1" alt=""></td>
	</tr>
        </table>
EOT
    }
    elsif (defined($style) and $style eq "reverse" ) {
        return ($right ? "$right&nbsp;" : "") . $left;
    }
    else {
        return ($left ? "$left&nbsp;" : "") . $right;
    }
}

sub _image {
        my %bd = %{shift(@_)};
        my %options = @_;
        my $alt = $bd{alt} || '';
        my $src = icon($bd{icon});
        return '' if not $src; # catch strange conditions
        my $extra;
        $extra .= " class=$options{class}" if $options{class};
        my $label = $bd{text} || "";
        return _button_format("<img src='$src' alt='$alt'>", $label, $options{class}, $options{style});
}

sub _link_img {
        my %bd = %{shift(@_)};
        my %options = @_;
        my $extra = "";
        my $src = icon($bd{icon});
        my $text = $bd{text} || '';
        $extra .= " class=$options{class}" if $options{class};
        $extra .= " target=$bd{target}" if $bd{target};
        if(defined($src) and $src) {
            return _button_format(
                "<a href='$bd{href}' $extra><img src='$src' alt='$bd{alt}' border=0 $extra></a>",
                "<a href='$bd{href}' $extra>$text</a>", $options{class}, $options{style});
        } else {
                return _link_text(\%bd,%options);
        }
}

sub _link_text {
        my %bd = %{shift(@_)};
        my %options = @_;
        my $extra;
        $extra .= " class=$options{class}" if $options{class};
        $extra .= " target=$bd{target}" if $bd{target};
        return _button_format("", "<a href='$bd{href}'$extra>$bd{text}</a>", $options{class}, $options{style});
}

sub _form_input_img {
        my %bd = %{shift(@_)};
        my %options = @_;
        
        my $alt = $bd{alt} || '';
        my $text = $bd{text} || '';
        my $extra;
        my $src = icon($bd{icon});
        $extra .= " class=$options{class}" if $options{class};
        my %text_opts = %options;
        $text_opts{style} = undef;
        $text = _form_text_input(\%bd,%text_opts) if $text;
        if (defined($src) and $src) {
            return _button_format(
                "<input type=image src='$src' name='$bd{name}' value='$bd{value}' alt='$alt' $extra>",
                $text, $options{class}, $options{style}
                );
        } else {
                return _form_input_button(\%bd,%options)
        }
}


sub _form_text_input {
        my %bd = %{shift(@_)};
        my %options = @_;
        my $value = defined($bd{value}) ? $bd{value} : $bd{name};
        my $id = $bd{slave} ? $bd{slave} : "$bd{name}_TEXT!!_";
        
        my $data;
        my $extra;
               
        $extra .= " class=$options{class}" if $options{class};
        $data .= "<SCRIPT LANGUAGE='JavaScript'>formClickElement('$bd{name}');</script>\n";
        $data .= "<input type=hidden name='$bd{name}' id='$id'>\n" unless $bd{slave};
        $bd{name} = $bd{slave} if $bd{slave}; # use the slaved name as the one to submit if it's passed
        $data .= qq|<a onMouseOver="window.status='$bd{text}'; return true" href="javascript:void(formClick('$bd{name}','$value'));" $extra>$bd{text}</a>\n|;
        return _button_format("", $data, $options{class}, $options{style});
}


sub _form_input_button {
        my %bd = %{shift(@_)};
        my %options = @_;
        my $value = $bd{value} || $bd{name};
        
        my $extra;
        $extra .= " class=$options{class}" if $options{class};
        return "<input type=submit name='$bd{name}' value='$value'$extra>\n";
}


=pod

=head1 NAME

PureMessage::Manager::Widget

=head1 SYNOPSIS

use PureMessage::Manger::Widget;

my $WIDGET = new PureMessage::Manager::Widget();

my $tb = $WIDGET->widget(
                CLASS =>'PureMessage::Manager::Widget::ToolBar',
                name => 'tb',
                buttons => [\@buttons],
                width=>600,
                state => \%state,
               );


print $tb->output();

=head1 DESCRIPTION

This package implements a base class for widget's and supplies some helpful
functions/methods to ease development.



=head1 METHODS

=over 4

=item new()

Creates a new widget object. Passed parameters are saved and accessible via 
the C<get> method.


=item widget(CLASS => $classname, %options)

Create and return a widget object of the specified CLASS. Any options
are passed unaffected to the constructor of the widget created.

=item errors( $error )

If $error is defined then pushes the passed error string onto the internal error
array otherwise returns the internal array as a list of error strings.

  $widget->errors("Something bad happened");

  if($widget->errors()) {
        print "The following errors occured: " . join("\n",$widget->errors());
  }


=item get( $keyname)

Retrieves the value of the requested key name. This method will first search the
user supplied configuration parameters for a value. If none is found it
will search the widget defaults for an appropriate value. If there is no default
value undef will be returned.  See the explanation of C<config_default> for
further information.

=item set( $key => $value )

Sets the passed user defined $key to the passed $value. This will be the value
that is returned by a C<$widget->get($key)> method call.

=item get_text( $data, %options )

This method attempts to resolve the passed $data parameter to a textual 
representation. The acceptable data types are:

=over 4

=item a PureMessage::Manager::Widget object

The return value will be the text that is returned by calling the output
method on the passed widget object.

=item ARRAY ref

The returned value is the concatenation of the C<get_text> output for 
each element in the array. If the options array has a C<post_row> key
it's value will be evaluated using C<get_text> and appended after each 
row element is evaluated.

=item HASH ref

The returned value is the concatenation of the C<get_text> output for
each value set in the hash. The order of elements is not defined. If
there is a C<post_row> key in the options array it's value will be
evaluated using C<get_text> and appended after each hash value is evaluated.

=item CODE ref

The returned value is the value that results from calling the passed
CODE reference passing %options as it's parameters. 

=item scalar

The text is returned verbatim.

=back

=item section_header

=item section_footer

Deprecated methods that reproduce a prior versions output for section
header/footers. Should not be used other than to facilitate porting
of code.

=item make_form_name( $element_name )

Widgets prepend some identifying data to form elements to help them
recognize CGI parameters that are directed at them. This method will 
return an element named C<$element_name> encoded in the style
appropriate for this widget to recognize as input for itself. The
resultant value should be used as the html form name.

This method depends on the user having passed a C<name> parameter
to the object constructor. If no C<name> paramater was set the
resultant element name will be recognized by any widget of the
same class, i.e. If widget is of type PageBar all PageBar widgets
will recognize the input as directed at them.

=item is_form_name ( $name )

If the passed C<$name> string matches the format as generated by
C<make_form_name> then this method will return an array containing
the widget name or class as the first element and the form element
name as the second.

  my ($widget,$form_element) = $WIDGET->is_form_name($cgi_input);
  print "Widget $widget revieved a form element called $form_element\n";

=item config_default( %defaults) 


Set the widget defaults for the passed keys. If the values are not
overwritten by constructor parameters or C<set> method calls then
the these values will be returned by C<get> method calls for the 
passed keys.

  $widget->set( a => 'a' );
  $widget->config_default( a=>'a2', b=>'b2');
  print $widget->get('a'); # will print 'a'
  print $widget->get('b'); # will print 'b2'

=item read_input( $REQUEST [,$widget_name] ) 

Will attempt to read the passed $REQUEST object decoding the input
names using C<is_form_name> and saving results into the INPUT
config setting.  If C<$widget_name> is defined then only form elements
that are recognized as being for the passed widget will be 
read into the INPUT data. If no input data was found then this
method will return false.

 if($widget->read_input($REQUEST,$TOOL_BAR_WIDGET_NAME )) {
        my $in = $widget->result_data();
 } else {
        // wasn't for this widget
 }


=item notification_header_str()

=item notification_footer_str()

Compatabilty method to ease porting of old code. Should not be used.

=item result_data()

Returns a hash reference that contains the results of the last 
read_input executed. Each key will have the same format as that
contained in a response object and will have any widget specific
form munging removed.

=item value_is_true( $value) 

Utility method that returns true if the passed value appears to be
a true value: yes|true|checked|ok|confirm|confirmed.

=item query( [$param] )

If param is passed will return the result_data value for that key
as was read by read_input. If C<$param> contains a non true value then
an array of all keys available is returned.

=item result_is_true()

Utility function that returns true if a cgi param of name RESULT contains
a value that C<value_is_true> determines is true. The RESULT param is 
used by several other widgets as the default to take action.

=item debug ($enable_debug)

If true sets internal debug flag to output debug data.

=item output_debug()

Returns html text dumping the state of the object. 

  print $widget->output_debug();

=back

=head1 FUNCTIONS

=over 

=item icon( $name )

Returns the location of the given icon C<$name> relative to the webroot. The
order of the search for the icon is: global /img directory, location of cgi
img directory, configured icons directory, or return a default /img/x.gif
image.

=item template ($name [ %options])

Return an HTML:Template object initialized with the passed option hash.

=item button( \%bd, %options )

Generate the html for a button of the passed type with the options listed.
This function will always return something that can be used to represent
the passed button. The format of the html generated is determined
by the existance of specific keys in the passed C<\%bd>.  If a button cannot
be generated using the passed settings then a best replacement is attempted
which may result in static text. For instance if an image type button is
requested but the icon doesn't exist then a text type button will be
substituted instead for it.

The possible keys in C<\%bd>are as follows:

=over 4

=item type

Determines what type of button to create and is required. Possible values are: 
image or icon, text, hidden, custom.

Note that this is used as a starting point for creation of a button. If for
some reason a button of the requested type cannot be generated a best guess
replacement will be returned instead.

=item name

The form name that this button should submit. Implies creation of a form element
button. The value of the button created can be set with the C<value> key. If the
value key doesn't exist then the name of this widget is used as the value.

=item href

The URL that the button will be linked to. Implies creation of a html anchor.

=item text

The text to display for this button. For image buttons this text will be placed
to the right of the image and may be omitted.  For text type buttons this will
be the text that is linked from.

=item icon

The URL of the image to display.

=item alt

For image type buttons this will be the alt text for the image.

=item target

The target of a href type button. This target that will be placed in the
generated html for links.

=item slave

The form name to use for text submits. 

Text type buttons are generated using custom javascript that requires that
there exist a hidden form element for communicating the buttons value when
the form is submitted. This form element is created with the value given
in the C<name> key for this button.

If you do not want this behaviour (for instance if you want to have multiple
buttons affect a change in a single form element then setting this key to
the name of an existing form element will cause the button to communicate
using that element instead of creating a new hidden one. 

=item value

For form type buttons (those that have the C<name> key set) this is the value
that will be returned when this button is activated. If this value is missing
for form type buttons then the button name will be used as the default value.

=back

The possible option keys are:

=over 4

=item class

The CSS class type for generated html.

=item style

The CSS style for generated html.

=back


=item state_buttons( [\@array|\%hash] )

Returns an array of hidden button definitions from the passed data, one
row for each key in the passed hash, the form name the same as the passed
key names. If the passed ref is to an array it is expected to be an
array of hashes and will generate a hidden button for each key in each row.

The formatted button definitions are formatted appropriately for feeding
directly to the C<buttons> function for translation to html.

=item format_state()

Returns html encoding the state that the widget has stored into the state
setting. This state setting is accessible via $widget->get('state') and
$widget->set('state') = \%hash;

=item buttons( \@bd, %options )

Returns the html for the passed button data as a single text scalar. The
C<%options> parameter is passed unchanged to the button function for each
button in the array.

 my @buttons = ( { type=>'icon',icon=>'save',text=>'Save', name=>'save'},
                 { type=>'icon',icon=>'cancel',text=>'Cancel', name=>'cancel'}
               );
 print buttons(\@buttons,class => 'button_class');


=item option_list( $name, \%data, $selected, [ $extra, \@order ])

A helper routine to generate an html select element with the passed
items and the C<$selected> item selected.  $name is the html form name
to use for the select, %data are the items to display in the list of 
possible selections, $selected is the key that should be displayed as
selected. $extra is html data that will be placed in the declaration
of the select. Order if given is the order the keys in %data should be
place in the list. If not given the order will default to alphabetic.


 my %day = ( 0 => 'Sun', 1 => 'Mon', 2 => 'Tue' , 3 => 'Wed' );
 # Mon is selected, Order is as above.
 print option_list('date',\%day,1,'class=stretch',qw( 0 1 3));


=back

=cut




1;

