package PureMessage::Manager::Widget::Dialog;
use PureMessage::Manager::Widget;
use Data::Dumper;
use strict;

our @ISA = qw(PureMessage::Manager::Widget);

sub new {
        my $CLASS = shift;
        $CLASS = ref($CLASS) || $CLASS;
        my $this = $CLASS->SUPER::new(@_);

        bless($this,$CLASS);
        return $this;
}

sub read_input {
  my $this = shift;
  my $request = shift;
  my $name = $this->get('name');
  $name = ref($this) unless $name;
  $this->SUPER::read_input($request,$name);
  if ($this->{INPUT}) {
    # then it was for us
    return 1;
  }
  return;
}

sub output {
        my $this = shift;
        my $data;
        my $style = $this->get('style');
        my $template;
        my %vars;
        my $w = new PureMessage::Manager::Widget;
        
        my %dialog_title = ( dlg_error=> 'Error',
                             dlg_info=> 'Information',
                           );
        $this->config_default( dialog_title => $dialog_title{$style});
        $this->config_default( name => ref($this));
        $this->config_default( width=> 500);
        
        my $name = $this->get('name');
        my $width = $this->get('width');
        
        # if buttons are supplied we will use them instead
        my @generic_buttons;
        if(my $buttons = $this->get('buttons')) {
                @generic_buttons = @$buttons;
        } else {
                @generic_buttons = ( [{ type=>'image', name=>'RESULT', 
                                          value=>'ok',icon=>'ok',text=>'Ok'
                                      },
                                     ]
                                   );
        }

        my %icons = ( dlg_error=> 'warning',
                      dlg_info=> 'info',
                      dlg_query => 'instructions',
                    );
        
        $vars{title} = $this->get('title');
        $vars{content} = $this->get('msg');
        $vars{width} = $width;
        $vars{dialog_title} = $this->get('dialog_title');
        $vars{dlg_icon} = icon($icons{$style} || 'info');

                   
        if($style =~ /dlg_info/i) {
        
          $template = template('_dialog');
          push(@{$generic_buttons[0]}, { type=>'hidden', name=>'dialog',
                                         value=>$style 
                                       },
              );
                                  
          my $buttons = $w->widget( CLASS => 'PureMessage::Manager::Widget::ToolBar',
                                    buttons => \@generic_buttons,
                                    action =>  $ENV{SCRIPT_NAME}, # send data back to me
                                    name => $name,
                                    embed => 1,
                                    state => $this->get('state'),
                                  );
          $vars{toolbar} = $buttons->output();
        
        } elsif($style =~ /dlg_error/i) {

          # error dialog has no buttons!
          $template = template('_dialog');
        
        } else {
          $vars{dialog_title} = 'Confirm' 
            if $style =~ /dlg_query/i and not $vars{dialog_title};
          $template = template('_dialog');
          if (my $buttons = $this->get('buttons')) {
                my $tb = $w->widget( CLASS => 'PureMessage::Manager::Widget::ToolBar',
                                     buttons => $buttons,
                                     action=>$ENV{SCRIPT_NAME},
                                     name => $name,
                                     embed=>1,
                                     state => $this->get('state'),
                                    );
                
                                        
                $vars{toolbar} = $tb->output() if $buttons;
          } elsif(my $tb = $this->get('toolbar')) {
                $tb->{width} = $vars{width};
                $tb->{embed} =1 unless $tb->{embed} == 0; # unless we are instructed not to
                $tb->{state} = $this->get('state');
                $vars{toolbar} = $tb->output();
          }
        }
       
        if ($template) {
          $template->param(\%vars);
          $data .= $template->output();
        } 
        return $data;
}
    
1;



=pod

=cut


=head1 NAME

PureMessage::Manager::Widget::Dialog

=head1 SYNOPSIS

 use PureMessage::Manger::Widget::Dialog;


 my $dlg = $WIDGET->widget(
                 CLASS =>'PureMessage::Manager::Widget::Dialog',
                 style => 'dlg_info',
                 msg => 'This is my Dialog,
                 title=>'Body Title,
                 buttons => [[ { type=>'image', text=>'Continue', icon=>'continue', href=>'index.cgi' } ]],
                 dialog_title=>'My dialog',     
            );  

 print $dlg->output();

=head1 DESCRIPTION

The Dialog widget object can be used to generate html for a
variety of standard dialogs.

=head1 METHODS

=over 4

=item new ( \%options ) 

Create a dialog object. Options are as follows:

=over 4

=item name

The name of this widget. Used to generate widget specific form input.

=item buttons

The buttons to display. This should be an array reference containing data
matching input expected by the C<buttons> function of Widget.
This data will be passed directly to the buttons function and used as the
action buttons on the generated dialog.

 my @buttons = ( { type=>text, text=>'Alrighty Then', name=>'ok' }, ... );

 my $dlg = new PureMessage::Manager::Widget::Dialog (
                buttons => [\@buttons,class => 'dlg_buttons'],
                ...
               );


=item style

The style of the dialog to generate. Valid styles are:

=over 4

=item dlg_info

Has an info icon and displays one button with the text of
ok. 

=item dlg_error

Has an error icon but no buttons.

=item dlg_query

By default has no buttons or content.

=item default

By default has no buttons but dialog_title defaults to 'Confirm'.

=back

=item title

The title to display in the body of the dialog.

=item dialog_title

The title to display in the frame of the displayed dialog.

=item msg 

The text to display in the dialog.


=item width

The width of the dialog frame.

=item toolbar

A toolbar widget to display on the dialog.

=item embed

if a toolbar or buttons decleration is supplied then setting this to
true means the form names for those items should be modified using
the C<make_form_name> function so that they are recognized as input for
this widget.  Default is to not do this.

=back

=item read_input( $request )

This item will take no actions but the DATA variable for this
widget (accessible via C<get_result>) will contain any button
data destined for this widget. Default buttons return a result
via the C<RESULT> form name.

=item result_data()

Returns a hash reference that contains the results of the last
read_input executed. Each key will have the same format as that
contained in a response object and will have any widget specific
form munging removed.

=item result_is_true()

Returns true if a confirmation button was executed from the
default button set. Having a button who's name is C<RESULT>
contain a true value when pressed will mimic the behaviour of
the default buttons.

=back

=cut




