package PureMessage::Manager::Widget::Form;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Lang;
use strict;

our @ISA = qw(PureMessage::Manager::Widget);

sub new {
        my $CLASS = shift;
        $CLASS = ref($CLASS) || $CLASS;
        my $this = $CLASS->SUPER::new(@_);
        bless($this,$CLASS);
        $this->config_default(name => $CLASS,
                              width => "100%",
                             );
        $this->{tmpl} = template("_form.html");
        return $this;
}

sub read_input {
  my $this = shift;
  my $request = shift;
  my $name = $this->get('name');
  $name = ref($this) unless $name;
  $this->SUPER::read_input($request,$name);
  if ($this->{INPUT}) {
    # then it was for us
    return 1;
  }
  return;
}


sub format_title {
        my $this = shift;
        my $cols = shift;
        my $title = $this->get('title');
        return ( TITLE => $title, TITLE_SPAN => $cols ) if $title;
        return;
}

sub getdd_col {
        my $this = shift;
        my $name = shift;
        my $ddata = $this->{'ddata'};
        return unless $ddata;
        return $ddata->{$name};
}


sub format_form_beginning {
  my $this = shift;
  my $action = shift;
  return unless $action;
  my $frm_prop = $this->{'form_properties'} || '';
  return "<form method=post name=" . $this->get('name') . " action=$action $frm_prop>";
}

sub format_form_close {
  my $this = shift;
  my $action = shift;
  return unless $action;
  return "</form>\n";
}

sub format_row {
        my $this = shift;
        my $data = shift;
        my $columns = shift;
        my @names = @_;
        my @result;
        my $keys = $this->get('keys');
        
        if (!$keys or ref($keys) ne 'CODE') {
                $keys = sub { return shift }; # set a default keys decorator if none
        } 
        my $wd1 = my $wd2 = int(100 / ($columns * 2));
        $wd1 /= 2; $wd2 += $wd1;
        for (my $index = 0; $index < $#names; $index++) {
                my $key_pretty;
                my $ddata = $this->getdd_col($names[$index]);
                $key_pretty = $ddata->{label};
                my $t = text($names[$index]);
                $key_pretty ||= &$keys($t);
                push @result, {
                                EXTRA   => "valign=top width='$wd1%' nowrap=yes",
                                CONTENT => $key_pretty,
                               };
                push @result, {
                                EXTRA   => "width='$wd2%'",
                                CONTENT => $data->{$names[$index]},
                               };
        }
        # then do the last row with maybe some colspan stretching
        my $cols = (($columns - @names) + 1) * 2 -1;
        my $index = $#names;
        my $key_pretty;
        my $ddata = $this->getdd_col($names[$index]);
        $key_pretty = $ddata->{label};
        my $t = text($names[$index]);
        $key_pretty ||= &$keys($t);
        push @result, {
                       EXTRA   => "valign=top width='$wd1%' nowrap=yes",
                       CONTENT => $key_pretty,
                      };
        push @result, {
                       EXTRA   => "width='$wd2%'",
                       CONTENT => $data->{$names[$index]},
                       COLSPAN => $cols,
                      };
        return \@result;
}

sub output {
        my $this = shift;
        my $result;
        my $data = $this->get('data');
        $data = &$data() if ref($data) eq 'CODE';
        my $style = $this->get('style');
        my $columns = $this->get('columns');
        my $fields = $this->get('fields') || [];
        my $extra_rows = $this->get('extra_rows') || [];
        my $w = new PureMessage::Manager::Widget;
        
        my $name = $this->get('name');

        my $width = $this->get('width');
        
        # we turn off the bottom border when there is a toolbar
        my $bottom_border = $this->get('toolbar') ? '0px' : '1px';
        
        $this->{tmpl}->param( WIDTH => $width );
        $this->{tmpl}->param( START => $this->format_form_beginning($this->get('action')) ) 
                if  $this->get('action');
        $this->{tmpl}->param( $this->format_title($columns * 2) );
        
        my @names;
        @names = @{$fields};
        @names = keys(%{$data}) unless scalar @names;
        my $i = 0;
        my @rows;
        while($i <= $#names) {
                my @cells;
                for(my $sub_i =0; $sub_i < $columns and $i <= $#names; $sub_i++) {
                       push(@cells, $names[$i]) if defined $names[$i];
                       $i++;
                }
                push @rows, { CELLS => $this->format_row($data,$columns,@cells) };
        }
        if (scalar @$extra_rows) {
            push @rows, @$extra_rows;
        }
        $this->{tmpl}->param( ROWS => \@rows );
        $result .= $this->format_state();
        if (my $tb = $this->get('toolbar')) {
                # take over this toolbar
                $tb->{name} = $this->get('name');
                $tb->{embed} =1;
                $tb->{width} = $this->get('width');
                my $colspan = int($columns/2);
                # if it's not embedded in a table then it is now :)
                #$tb->{table} = [[$colspan,$columns-$colspan],['40%','60%']] unless $tb->{table}; 
                $result .= $tb->output();
        }
        $this->{tmpl}->param( TOOLBAR=>$result );
        $result = $this->get('action') ? $this->format_form_close($this->get('action')) : "";
        $result .= $this->get('padding') if $this->get('padding');
        $this->{tmpl}->param( END=>$result );
        return $this->{tmpl}->output();
}
    
    
1;



