package PureMessage::Manager::Widget::Grid;
use PureMessage::Manager::Widget;
use strict;

our @ISA = qw(PureMessage::Manager::Widget);

sub new {
        my $CLASS = shift;
        $CLASS = ref($CLASS) || $CLASS;
        my $this = $CLASS->SUPER::new(@_);
        bless($this,$CLASS);
        $this->config_default(name => $CLASS,
                              width => "100%",
                              page => 0,
                              page_size => 1000,
                              paging=>0,
                             );
        $this->{tmpl} = template("_grid.html");
        return $this;
}

sub read_input {
  my $this = shift;
  my $request = shift;
  my $name = $this->get('name');
  $name = ref($this) unless $name;
  $this->SUPER::read_input($request,$name);
  return $this->handle_events();
}

sub handle_events {
  my $this = shift;
  if ($this->{INPUT}) {
    # then it was for us
    my $in = $this->{INPUT};
    if($in->{current_page}) {
        $this->{page} = $in->{current_page};
        delete($this->{INPUT}->{current_page});
    }
    if ($in->{page_next}) {
        $this->{page} = $this->{page} +1;
        $this->{INPUT}->{new_page} = $this->{page};
        delete($this->{INPUT}->{page_next});
    }
    if ($in->{page_last}) {
        $this->{page} = $this->{page} -1;
        delete($this->{INPUT}->{page_last});
        $this->{INPUT}->{new_page} = $this->{page};
    }
    if (defined($in->{goto_page}) and $in->{goto_page} =~ /\d+/) {
        $this->{INPUT}->{new_page} = $in->{goto_page} if $this->{page} != $in->{goto_page};
        $this->{page} = $in->{goto_page};
        delete($this->{INPUT}->{goto_page});
    }
    if($in->{edit_selected} and defined($in->{selected_row})) {
      my @selected;
      if (ref($in->{selected_row}) eq 'ARRAY') {
        @selected = @{$in->{selected_row}};
      } else {
        @selected = ($in->{selected_row});
      }
      my @grid_edit;
      foreach my $index (@selected) {
        next if $index == -1; # ignore the special row
        push(@grid_edit,$index);
      }
      $this->{edit} = \@grid_edit;
      $this->{selectable} = 0;
      $this->{use_alt_special} =1;
    } elsif($in->{edit_selected}) {
      # they clicked edit but didn't select a column
      delete $in->{edit_selected};
      delete $this->{INPUT}->{edit_selected};
    }
 
    if($in->{save_editable}) {
          my @fields = sort grep /grid_edit/, keys %$in;
          my (%result,%temp);
          if (@fields) {
                foreach my $field (@fields) {
                        my ($junk,$row,$column) = split(':',$field);
                        next unless $junk eq 'grid_edit';
                        $temp{$row}->{$column} = $in->{$field};
                }
                foreach my $key (keys %temp) {
                        my @columns;
                        foreach my $c (sort keys %{$temp{$key}}) {
                                push(@columns,$temp{$key}->{$c});
                        }
                        $result{$key} = \@columns;
                }
                
                $this->{save_editable} = \%result;
          }
    }

    if (defined($in->{selected_row})) {
      # then supply a translated version to absolute numbers so that we account
      # for paging
      my @selected;
      if (ref($in->{selected_row}) eq 'ARRAY') {
        @selected = @{$in->{selected_row}};
      } else {
        @selected = ($in->{selected_row});
      }
      foreach my $row (@selected) {
        $row = $this->translate_row($row) unless $row == -1;
      }
      $in->{selected_data_rows} = \@selected;
    }
    my @keys = keys(%{$this->{INPUT}});
    return $#keys >= 0;
  }
  return;
}

sub translate_row {
  my $this = shift;
  my $index = shift(@_) + ($this->{page} * $this->get('page_size'));
  return $index;
}

sub edit_result {
        my $this = shift;
        return $this->{save_editable};
}

# using the current settings for page page_size calculate the starting and
# ending range of array elements that should be displayed
sub translate_index {
        my $this = shift;
        my $count = $this->_get_data_mindex() + 1;
        # do the get max page dance
        my $max_page = int($count / $this->get('page_size'));
        $max_page++ if ($max_page * $this->get('page_size')) < $count;
        $max_page = $max_page -1 > 0 ? $max_page -1 : 0; #list is zero based, and ignore page 0 
        
        # catch invalid pages
        $this->{page} = $max_page if $this->get('page') > $max_page;
        my $start = $this->get('page') * ($this->get('page_size'));
        my $end = $start + $this->get('page_size') -1;
        return ($start,$end);
}

sub format_row {
        my $this = shift;
        my $index = shift;
        my @row = @_;
        my @result;
        my $edit = $this->get('edit_lines');
        my $name = $this->make_form_name('selected_row');
        if ($this->get('selectable')) {
            push @result, {
                            EXTRA   => " width=10 ",
                            CONTENT => "<input type=checkbox name='$name' value=$index>"
                           };
        }
        
        my $i = 0;
        while($i <= $#row) {
                my $col = $row[$i];
                my $data = $this->getdd_col($i);
                if($data and $data->{decorate}) {
                        if(ref($data->{decorate}) eq 'CODE') {
                                $col = &{$data->{decorate}}($col,$index);      
                        }
                        # else do nothing
                
                }
                my $extra = $data->{extra} || '';
                $extra .= $data->{width} ? " width=$data->{width} " : '';
                if ($edit->{$index}) {
                        $this->{ADD_SAVE} = 1;
                        push @result, {
                                        EXTRA   => $extra,
                                        CONTENT => "<input type=text width=20 name='" 
                                                    .  $this->make_form_name("grid_edit:$index:$i")
                                                    .  "' value='"
                                                    .  $col
                                                    .  "'>"
                                       };
                        $i++;
                } else {
                        # collect undef's and turn them into a stretched colspan
                        my $colsp = 0;
                        $colsp++ while $i <= $#row and $row[++$i] eq undef;
                        push @result, {
                                        EXTRA   => $extra,
                                        CONTENT => $col,
                                        COLSPAN => $colsp,
                                       };
                }
                
        }
        return \@result;
}

sub format_title {
        my $this = shift;
        my $cols = shift;
        $cols++ if $this->get('selectable'); # add a column if we are adding checkboxes
        my $title = $this->get('title');
        return ( TITLE => $title, TITLE_SPAN => $cols ) if $title;
        return "";
}

sub getdd_col {
        my $this = shift;
        my $index = shift;
        my $ddata = $this->get('ddata');
        return unless $ddata;
        return $ddata->[$index];
}

sub format_headers{
    my $this = shift;
    my $cols = shift;
    return unless $cols and $cols > 0;
    my @result;
    if(my $ddata = $this->get('ddata')) {
        # we have some data descriptors
        for(my $i=0; $i < $cols; $i++) {
            my $desc = $ddata->[$i];
            # if we have checkboxes then make the first title span
            # two columns
            my $extra = "";
            $extra .= ' colspan=2 ' if $i ==0 and $this->get('selectable');
            $extra .= " width='$desc->{width}' " if ($desc->{width});
            if ($desc and $desc->{'description'}) {
                push @result, { EXTRA   => $extra, 
                                CONTENT => $desc->{description}, };
            }
            else {
                push @result, { EXTRA   => $extra, 
                                CONTENT => "&nbsp;", };
            }
        }
    }
    # else do nothing
    return \@result;
}

sub format_special {
    my $this = shift;
    my $special = $this->get('special');
    return unless $special;
    $special = $this->get('edit_special') if $this->{use_alt_special} and $this->get('edit_special');
    my @result;
    my $name = $this->make_form_name('selected_row');
    
    if ($this->get('selectable')) {
        push @result, { EXTRA   => " width=10 ", 
                        CONTENT => "&nbsp;", };
        #push @result, { EXTRA   => " width=10 ", 
        #                CONTENT => "<input type=checkbox name='$name' value=-1>", };
    }
    
    for my $row (@$special) {
        push @result, { CONTENT => $row, };
    }
    return \@result;
}

sub format_form_beginning {
  my $this = shift;
  my $action = shift;
  return unless $action;
  return "<form method=post name=" . $this->get('name') . " action=$action>";
}

sub format_form_close {
  my $this = shift;
  my $action = shift;
  return unless $action;
  return "</form>\n";
}

sub format_paging {
  my $this = shift;
  my $count = shift(@_) + 1; #last valid index + 1
  my ($max_page,$current_page);
  $max_page = int($count / $this->get('page_size'));
  $max_page++ if ($max_page * $this->get('page_size')) < $count;
  $max_page = $max_page -1 > 0 ? $max_page -1 : 0; #list is zero based, and ignore page 0 
  $current_page = $this->get('page');
  my $widget = $this->widget(CLASS => 'PureMessage::Manager::Widget::PageBar',
                             max_page => $max_page,
                             current_page => $current_page,
                             embed => 1,
                             item_count => $this->get('show_count') ? $count : undef,
                             name => $this->get('name'),
                            );
  $this->{paging} = $widget;
  return $widget->output();
                             
}
  
sub _get_data_mindex {
        my $this = shift;
        my $data = $this->get('data');
        my $mindex = 0;
        if (ref($data) eq 'CODE') {
                $mindex =  &$data();
        } elsif(ref($data) eq 'ARRAY') {
                $mindex = $#$data >= 0 ? $#$data : -1;
        }
        return $mindex;
}

# note that because translate index may have changed the current page if it was
# invalid, all external references to row numbers will be wrong. This means
# that if you are going to manipulate selected_rows you *must* do so before
# calling the output method
#
sub output {
        my $this = shift;
        my $result;
        my $data = $this->get('data');
        my $style = $this->get('style');
        my $columns = $this->get('columns');
        
        my $w = new PureMessage::Manager::Widget;
        
        my $name = $this->get('name');
        my $width = $this->get('width');

           # what index the user *thinks* these rows are
        my ($start,$end) = $this->translate_index();
           # the actual index we are going to show from our data array
        my ($real_start,$real_end) = ($start,$end);
        
        my $mindex;
        
        if(my $edit_rows = $this->get('edit')) {
                my %edit = map { $_ => 1 } @{$this->get('edit')};
                $this->{edit_lines} = \%edit;
        }
        
        if (ref($data) eq 'CODE') {
                # temp
                $mindex = &$data();
                $data = &$data($start,$end);
                if ($mindex == $#{@$data} and $#{@$data} >= 0) {
                        # as a special case if the amount of data returned is
                        # the same as the mindex then we will assume that they
                        # have returned all the data and we are responsible for
                        # paging
                        $real_start = $start;
                        $real_end = $end;
                } else {
                        # we have to adjust our absolute indexes because we only have
                        # this many elements
                        $real_start = 0;
                        $real_end = $#{@$data};
                }

        } elsif(ref($data) eq 'ARRAY') {
                $mindex = $#$data;
        }
        $columns = $#{@{$data->[0]}} unless $columns;
        my $bottom_width;
        $bottom_width = $this->get('toolbar') ? 0 : 1;
        $this->{tmpl}->param( WIDTH => $width );
        $this->{tmpl}->param( START => $this->format_form_beginning($this->get('action')) ) 
                if  $this->get('action');
        $this->{tmpl}->param( $this->format_title($columns) );
        $this->{tmpl}->param( HEADER => $this->format_headers($columns) );
        my @rows;
        my $special = $this->format_special($columns);
        push @rows, {CELLS => $special} if $special;
        if ($real_start <= $mindex) {
            for(my $i=$real_start; $i <= $mindex and $i <= $real_end; $i++) {
                push @rows, {CELLS => $this->format_row($i,@{$data->[$i]})};
            }
        } else {
            # don't display anything because we are past the end of
            # our data.
        }
        
        $columns++ if $this->get('selectable');
        
        # paging
        if ($this->get('paging') and $this->get('page_size') ) { 
            push @rows, {CELLS => [ {CONTENT=>$this->format_paging($mindex), COLSPAN=>$columns}] };
        }
    
        $this->{tmpl}->param( ROWS => \@rows );

        # toolbar
        if (my $tb = $this->get('toolbar')) {
                if (ref($tb) eq 'CODE') {
                    $tb = &$tb();
                }
                # take over this toolbar
                $tb->{name} = $this->get('name');
                $tb->{embed} =1;
                $tb->{width} = $this->get('width');
                # if it's not embedded in a table then it is now :)
                #$tb->{table} = [[$colspan,$columns-$colspan],['40%','60%']] unless $tb->{table}; 
                if($this->get('editable')) {
                        if($this->{ADD_SAVE}) {
                        push(@{$tb->{buttons}->[0]},{ type=>'image',
                                                      name=>'save_editable',
                                                      icon=>'save',
                                                      text=>'Save',
                                                    });
                        } else {
                         push(@{$tb->{buttons}->[0]},{ type=>'image',
                                                      name=>'edit_selected',
                                                      icon=>'edit',
                                                      text=>'Edit',
                                                    });
                        }
                }
                $result .= $tb->output();
        }
        $result .= $this->format_state($this->get('state')) if $this->get('state');
        $this->{tmpl}->param( TOOLBAR=>$result );
        $result = $this->get('action') ? $this->format_form_close($this->get('action')) : "";
        $result .= $this->get('padding') if $this->get('padding');
        $this->{tmpl}->param( END=>$result );
        return $this->{tmpl}->output();
}
    
    
1;



