package PureMessage::Manager::Widget::ListEditor;
use PureMessage::Manager::Widget;
use PureMessage::ListManager;
use PureMessage::ListFile;
use PureMessage::List;
use PureMessage::Map;
use PureMessage::Manager::Lang qw(&text);
use PureMessage::Manager::Lang;
use PureMessage::Resources::Manager;
use PureMessage::Manager::Widget::Dialog;
use HTML::Entities; 
use strict;

use base qw(PureMessage::Manager::Widget);

sub new {
        my $CLASS = shift;
        $CLASS = ref($CLASS) || $CLASS;
        my $this = eval {bless $CLASS->SUPER::new(@_),$CLASS};
        if ($@) {
                # then there was an error loading the list
                $this->{ERROR_LOADING_LIST} = $@;
                return PureMessage::Manager::Widget::Dialog->new( style => 'dlg_info',
                                                           msg => "$@", 
                                                           title=>text('list_init_died'),
                                                           buttons => [[ { type=>'image', text=>'Continue', icon=>'continue', href=>'index.cgi' } ]],
                                                         );           
        } else {
                $this->_init_widgets();
        }
        return $this;
}

sub _init {
  my $this = shift;
  $this->config_default(name  => ref($this),
                        width => "500",
                        css_class_1 => 'medium',
                        css_class_2 => 'light',
                        page_size   => 11,
                        filter      => '',
                        ddata       => [{ description => 'List Items'}],
                        grid_columns=> 1,
                        selectable => 1,
                        );
                        
  $this->config_default(name  => ref($this),
                        toolbar_width => $this->get('width'),
                        left_width    => int($this->get('width')*0.4),
                        right_width   => $this->get('width')-$this->get('left_width'),
                       );

  $this->config_default(toolbar_width => $this->get('left_width'),
                        filter_width  => $this->get('left_width'),
                        filter_table_width => $this->get('left_width'),
                       );
  $this->config_default(grid_toolbar  => sub {$this->widget(
                                                        CLASS => 'PureMessage::Manager::Widget::ToolBar',
                                                        embed =>1,
                                                        name => $this->get('name'),
                                                        buttons=> $this->_get_grid_toolbar_buttons(),
                                                       )});


  my $rm = PureMessage::Resources::Manager->new();
  my $lr = $rm->find($this->get('list'));
  my $l;
  if ($lr) {
      $l = $lr->native();
      $this->{is_resource} = $rm->store_type();
  } else {
      my $lm = PureMessage::ListManager->new( fetch => [qw(List)]);
     $l = $lm->find_list($this->get('list'), writable => 1,edit_mode => 1);
  }
  
$this->{LIST_INFO} = $l;
  $this->{LIST_TYPE} = 'list';
  if($l) {
  
          $this->{LIST} = PureMessage::rListWrapper->new($this->get('list'),$l);

  } else {
          die "Unable to create list object";
  }
  $this->_prep_list();
}

sub _get_grid_toolbar_buttons {
        my $this = shift;
        my @buttons =  ();
        my $q = $this->result_data();
        if (not $q->{'edit_selected'} or !($q->{'edit_selected'} and defined($q->{'selected_row'}))) {
            return [[{ type=>'image',
                       icon=>'delete',
                       text=>'Delete',
                       name=>'delete_selected',
                   }]];
        }
        else {
            return [[{ type=>'image',
                       icon=>'cancel',
                       text=>'Cancel',
                       name=>'cancel',
                   }]];
        }
}



sub _prep_list_item {
  my $this = shift;
  return [shift];
}
sub _prep_list {
  my $this = shift;
  my $filter = shift;
  my $sort = shift;
  my %raw_list;
  %raw_list = $this->{LIST}->entries(1); # keys and data
  my %cooked_list;
  my @cooked_entries;
  foreach my $key (keys %raw_list) {
    next unless $raw_list{$key} =~ /./; # at least one char
    push(@cooked_entries,$this->_prep_list_item($raw_list{$key}));
    next if $filter and $raw_list{$key} !~ /$filter/oi;
    $cooked_list{$key} = $raw_list{$key};
  }
  %raw_list = undef;
  my @final_list;
  my @keys = keys %cooked_list;
  @keys = sort { $cooked_list{$a} cmp $cooked_list{$b}} @keys;
  foreach my $key ( @keys ) {
    my $value = $cooked_list{$key};
    encode_entities($value, '<>'); 
    push(@final_list,{ line => $key, value=> $this->_prep_list_item($value) });
  }
  $this->{LIST_DATA} = \@final_list;
  $this->{ALL_LIST_DATA} = \@cooked_entries;
  $this->{LIST_SIZE} = $#final_list >= 0 ? $#final_list : -1;
}


sub _init_widgets {
        my $this = shift;
        $this->{'GRID'} = $this->widget( CLASS => 'PureMessage::Manager::Widget::Grid',
                                          page_size => $this->get('page_size'),
                                          page => 0 ,
                                          selectable => $this->get('selectable'),
                                          columns=>$this->get('grid_columns'),
                                          name => $this->get('name'),
                                          embed => 1,
                                          editable => 1,
                                          width => $this->get('toolbar_width'),
                                          ddata => $this->get('ddata'),
                                          special => $this->get('special'),
                                          edit_special => [undef,undef],
                                          toolbar => $this->get('grid_toolbar'),
                                          paging => 1,
                                          data => sub {
                                                        my ($start,$end) = @_;
                                                        my $size = $#{@{$this->{LIST_DATA}}};
                                                         if (defined($start) and defined($end) and $start <= $size) {
                                                          my @data;
                                                          for(my $i = $start; $i <= $end and $i <= $size; $i++){
                                                            push(@data,$this->{LIST_DATA}->[$i]->{value});
                                                          }
                                                          return \@data;
                                                         } elsif($start == undef and $end == undef) {
                                                                return $this->{LIST_SIZE};
                                                         } else {
                                                                return [];
                                                         }


                                                      },
                                                                                    
                                        );

}


sub _format_entry {
        my $this = shift;
        return @{shift(@_)};
}

sub read_input {
  my $this = shift;
  my $request = shift;
  return if $this->{ERROR_LOADING_LIST};
  my $name = $this->get('name');
  my %in;
  $name = ref($this) unless $name;
  $this->SUPER::read_input($request,$name);
  $this->{GRID}->read_input($request,$name);
  
  
  if ($this->{INPUT}) {
  
      %in = %{$this->{INPUT}};
      
      if($in{start_filter_by}) {
        $this->{GRID}->{page} = 0;
        $this->{filter} = $in{filter_by};
        $this->{GRID}->{page} = 0 if $in{filter_by} ne $in{displayed_filter_by};
        delete($this->{INPUT}->{start_filter_by});
        delete($this->{INPUT}->{filter_by});
        delete($this->{INPUT}->{displayed_filter_by});
        delete($in{displayed_filter_by});
      }

      if($in{displayed_filter_by}) {
        $this->{filter} = $in{displayed_filter_by};
        delete($this->{INPUT}->{displayed_filter_by});

      }
  }

  if(my $data = $this->{GRID}->edit_result()) {
        # then something that was edited should be saved
    $this->_prep_list($this->{filter});
    
    foreach my $index (keys %$data) {
      next if $index == -1; # ignore the special row
      my $line_number = $this->{LIST_DATA}->[$this->translate_row($index)]->{line};
      if ($this->{LIST_TYPE} eq 'list') {
        if($this->validListItem($this->_format_entry($data->{$index}),1)) {
                $this->{LIST}->entry($line_number,$this->_format_entry($data->{$index}));
                delete($data->{$index});
		$this->{dirty} = 1;
        }# else there will be an error thrown
      } else {
        if($this->validMapItem($data->{$index}->[0],$data->{$index}->[1],1)) {
                $this->{LIST}->entry($line_number,@{$this->_format_entry($data->{$index})});
                delete($data->{$index});
		$this->{dirty} = 1;
        }
      }
    }

    $this->{LIST}->commit();
  }
  
  if ($this->{INPUT}) {
    return $this->handle_events();
  }
  return;
}

sub translate_row {
  my $this = shift;
  my $index = shift(@_) + ($this->{GRID}->{page} * $this->get('page_size'));
  return $index;
}

sub handle_events {
  my $this = shift;
  my %in = %{$this->{INPUT}};
  if($in{delete_selected} and defined($in{selected_row})) {
    $this->_prep_list($this->{filter});
    my @selected;
    if (ref($in{selected_row}) eq 'ARRAY') {
      @selected = @{$in{selected_row}};
    } else {
      @selected = ($in{selected_row});
    }
    foreach my $index (@selected) {
      next if $index == -1; # ignore the special row
      my $line_number = $this->{LIST_DATA}->[$this->translate_row($index)]->{line};
      $this->{LIST}->delete($line_number);
      $this->{LIST_SIZE}--;
      $this->{dirty} = 1;
    }
    $this->{LIST}->commit();
  }
  

  if($in{do_bulk_add} and defined($in{bulk_list_input})) {
    # have to read the list in this instance so that we can check for
    # duplicates
    $this->_prep_list($this->{filter});
    my @items = split(/\r?\n/,$in{bulk_list_input});
    my @bad;
    foreach my $item (@items) {
      next unless $item =~ /./;
      chomp($item);
      $item =~ s/^\s+//;
      $item =~ s/\s+$//;
      if ( ! $this->validListItem($item) ) {
        push(@bad,$item);
        next;
      } else {
        $this->{LIST}->append($item);
	$this->{dirty} = 1;
      }
    }
    $this->{LIST}->commit();
    $this->set('redisplay_bulk',join("\n",@bad)) if @bad;
  }
  if($in{add_new} and defined($in{add_in1}) and defined($in{add_in2})) {
    $this->_prep_list($this->{filter});
    my @val = ($in{add_in1},$in{add_in2});
    foreach (@val) {    
        chomp($_);
        s/^\s+//;
        s/\s+$//;
    }
    if ($this->validMapItem(@val) ) {
        eval { 
          my $mapvals = $this->_format_entry(\@val);
          foreach my $item (@$mapvals) {
              $this->{LIST}->append($item);
          }
          $this->{LIST}->commit();
	  $this->{dirty} = 1;
        };
        if($@) {
          $this->errors("Error Committing List data: $@");
        } else {
          $this->_prep_list($this->{filter});
        }

    } else {
        $this->set('redisplay_add',\@val);
    }
  }

}

sub _list_test {
    my $this = shift;
    my $item = shift;
    my $list;
    my %i = %{$this->{LIST_INFO}};
    $item = ref($item) eq 'ARRAY' ? $item : [$item];
    eval {
        my %data = (  description => $i{description},
                      items => $item,
                      name => $i{name},
                      id => $i{id},
                      match_type => $i{match_type},
                      comparator => $i{comparator},
                   );
        $list = new PureMessage::List(%data);
        $list->match(@$item);
        
    };
    if ($@) {
        $this->errors(&text('bad_syntax',@$item,$@));
        return;
    } else {
        return 1;
    }
}

sub _map_test {
    my $this = shift;
    my $key = shift;
    my $value = shift;
    my $list;
    my %i = %{$this->{LIST_INFO}};
    my $item = [ [$key,$value] ];
    eval {
        my %data = (  description => $i{description},
                      items => $item,
                      name => $i{name},
                      id => $i{id},
                      match_type => $i{match_type},
                      comparator => $i{comparator},
                   );
        $list = new PureMessage::Map(%data);
        
        $list->transform($key);
        
    };
    if ($@) {
        $this->errors(&text('bad_syntax' . $@,@$item,$@));
        return;
    } else {
        return 1;
    }
}



sub validMapItem {
    my $this = shift;
    my ($key,$value,$dups) = @_;
    my $ERROR = 0;
    $ERROR = $this->errors(&text('duplicate_map',$key,$value)) || 1 if !$dups and grep { $key eq $_->[0] } @{$this->{ALL_LIST_DATA}};
    $ERROR .= $this->errors(&text('empty_map_key')) || 1 unless $key =~ /[^\s]+/;
    #$ERROR .= $this->errors(&text('empty_map_list')) unless $value =~ /[^\s]+/;
    $ERROR .= 1 unless $this->_map_test($key,$value);
    return !$ERROR;
    
}

sub validListItem {
    my $this = shift;
    my $item = shift;
    my $dups = shift;
    my $ERROR = 0;
    $ERROR = $this->errors(&text('duplicate_list_entry',$item)) || 1 if !$dups and grep { $item eq $_->[0]} @{$this->{ALL_LIST_DATA}};
    $ERROR = 1 unless $this->_list_test($item);
    return !$ERROR;
}

sub _start_form {
  my $this = shift;
  return '<form method=post action="' . $this->get('action') . '">';
}
 
sub _end_form {
  my $this = shift;
  return '</form>';
}


sub _format_filter {
  my $this = shift;
  my $result;
  my @buttons = ({ type=>'hidden',
                   name=>$this->make_form_name('displayed_filter_by'),
                   value=>$this->get('filter'),
                 },
                 { type=>'custom',
                   text => '<input type=text size=10 name="' 
                           . $this->make_form_name('filter_by') 
                           . '" value="'
                           . $this->get('filter')
                           . '">',
                 },
                 { type=>'image',
                   name=>$this->make_form_name('start_filter_by'),
                   icon=>'filter',
                   text=>'Filter',
                });

  my @bd = buttons(\@buttons);
  my $filter = join('&nbsp;',@bd);
  $filter =<<EOF;
        $bd[0]
        <table cellpadding=2 cellspacing=0>
        <tr>
          <td>$bd[1]</td>
          <td nowrap="nowrap">$bd[2]</td>
        </tr>
        </table>
EOF

  my $data;
  $data = "<table cellpadding=0 cellspacing=0 width='" . $this->get('filter_table_width') . "'>\n";
  $data .= "<tr><td align=right valign=top >\n";
  $data .= $this->widget( CLASS => 'PureMessage::Manager::Widget::PaneBox',
                           content=>$filter,
                           width=> $this->get('filter_width'),
                           suppress_bottom_border=>1,
                           td_extra=>'align=right valign=top nowrap',
                         )->output;
  $data .= "</td></tr>\n";
  return $data;                          
}



sub _format_bulk_input {
  my $this = shift;
  my $content = <<EOF;
  <b>Add Items</b><p>
 To add one or more items enter them below, each on a separate line.<p> 
EOF
  my $BULK_INPUT = $this->get('redisplay_bulk');
  $this->errors("BULK: $BULK_INPUT");
  $content .= "<textarea name='" 
            . $this->make_form_name('bulk_list_input')
            . "'cols=25 rows=15 style='width: 100%'>$BULK_INPUT</textarea>\n";

  return $this->widget( CLASS => 'PureMessage::Manager::Widget::PaneBox',
                        content=>$content,
                        width=>$this->get('right_width'),
                        toolbar => $this->widget( CLASS => 'PureMessage::Manager::Widget::ToolBar',
                                                  name => $this->get('name'),
                                                  embed =>1,
                                                  buttons=> [[
                                                            { type=>'image',
                                                              icon=>'add',
                                                              text=>'Add',
                                                              name=>'do_bulk_add',
                                                              value=>'add',
                                                            },
                                                            { type=>'image',
                                                              icon=>'clear',
                                                              text=>'Clear',
                                                              name=>'do_bulk_clear',
                                                            }
                                                            ]],
                                                ),
                      )->output;
}

sub putErrors {
    my $this = shift;
    if (my @errors = $this->errors()) {
            my $err = $this->widget('CLASS'=>'PureMessage::Manager::Widget::Error',
                                    error => \@errors,
                                   );
            return $err->output();       
    }
    return '';

}


sub output {
        my $this = shift;
        my $result;
        my $data = $this->get('data');
        my $style = $this->get('style');
        my $result;
        if ($this->{ERROR_LOADING_LIST}) {
                $this->errors(&text('list_init_died',$this->{ERROR_LOADING_LIST}));
                return $this->putErrors();
        }
	
        if ($this->{dirty}) {
            if ($this->{is_resource}) {
        	    $result .= $this->_info_box(text('list_is_resource',$this->{is_resource})) ;
                $result .= '<BR/>' . $this->_info_box(text('resource_list_is_dirty'));
            }
            else {
                $result .= $this->_info_box(text('list_is_dirty')); 
            }
        }

        $this->_prep_list($this->get('filter'));
        my $w = new PureMessage::Manager::Widget;
        
        my $name = $this->get('name');

        my $width = $this->get('width');
        $result .= $this->putErrors();
        $result .= $this->_start_form();
        $result .= "<table cellpadding=0 cellspacing=0>";
        $result .= "<tr><td valign=top>";
        $result .= $this->_format_filter();
        $result .= $this->{GRID}->output;
        $result .= "</td>\n";
        $result .= "<td width=10>&nbsp;</td>\n";
        $result .= "<td valign=top>";
        $result .= $this->_format_bulk_input();
        $result .= "</td></tr></table>\n";
        if ($this->get('state')) {
            my $state = $this->get('state');
            # if it's a hash disallow the restartDlg from displaying since this
            # widget is incompatable with it
            $state->{query_ok} = 0 if ref($state) eq 'HASH';
            $result .= $this->format_state($state);
        }
        $result .= $this->_end_form();
        return $result;
}
 
sub _info_box {
    my ($self, $msg) = @_;
    my $ib = $self->widget(
	CLASS => 'PureMessage::Manager::Widget::InfoBox',
	msg => $msg);
    return $ib->output;
}
1;

package PureMessage::rListWrapper;
use PureMessage::Resources::Manager;

sub new {
  my $CLASS = shift;
  my ($list_name,$lo) = @_;
  $CLASS = ref($CLASS) || $CLASS;

  my $ld = $lo->data();
  if ($ld->is_readonly ) {
     die "List is readonly '$list_name'";
  }
  my $self = { list => $list_name,ldata => $ld };
  bless($self,$CLASS);
  $self->_entries();
  return $self;
}

sub entries {
  my ($self,$index) = @_;
  return %{$self->{entries}} if $index;
  return scalar(keys %{$self->{entries}}) unless wantarray;
  return map { $self->{entries}->{$_} } keys %{$self->{entries}};
}

sub _entries {
  my ($self,$do_index) = @_;
  # get all the entries
  my @data;
  $self->{ldata}->enumerate_items( sub { my $item = shift; return unless length($item); push(@data,"$item")});
  my %entries;
  my $index;
  foreach my $line (@data) {
    $entries{$index++} = $line;
  } 
  $self->{entries} = \%entries;
  $self->{entries_count} = $index;
  return %entries if $do_index;
  return scalar(@data) unless wantarray;
  return @data; 
}

sub entry {
  my ($self,$lnum,$newvalue) = @_;
  my $value = $self->{entries}->{$lnum};
  $self->{entries}->{$lnum} = $newvalue if defined($newvalue); 
  return $value;
}

sub delete {
  my ($self,@lines) = @_;
  delete($self->{entries}->{$_}) foreach @lines;
}

sub append {
  my ($self,@lines) = @_;
  $self->{entries}->{$self->{entries_count}++} = $_ foreach @lines;
}

sub commit {
  my $self = shift;
  $self->{ldata}->clear();
  my @items = map { $self->{entries}->{$_} } sort keys %{$self->{entries}};
  $self->{ldata}->add_items(@items);
  $self->{ldata}->commit();
}

1;

=pod

=cut


=head1 NAME

PureMessage::Manager::Widget::ListEditor

=head1 SYNOPSIS

 use PureMessage::Manger::Widget::ListEditor;

 my $WIDGET = new PureMessage::Manager::Widget();

 my $l = $WIDGET->widget(
                   CLASS =>'PureMessage::Manager::Widget::ListEditor',
                   name => 'edit_list',
                   list => $list_id,
                   page => 0,
                   action => $ENV{SCRIPT_NAME},
                   state => { id => $list_id},
                );

 $l->read_input();

 print $l->output();

=head1 DESCRIPTION

The ListEditor widget object can be used to generate html for a
editor for C<PureMessage::List>s.

=head1 METHODS

=over 4

=item new

Available options are:

=over 4

=item action

The value that should be used in the form action paramater. 

=item filter

A regular expression that will be used to filter the displayed
list items.

=item list

ListID of the list to edit. 

=item name

Name of this widget.

=item page_size

How many list items to display per page.

=item state

A hash reference of items that should be encoded as hidden form
elements.

=item width

The width of the widget to display.

=item page

The page of list contents to display. This will be modified by the
C<read_input> method and should only be used when forcing display
of a specific page.

=back

=item read_input( $RESPONSE )

Read the response input. This will manage all aspects of list editing.


=back



