package PureMessage::Manager::Widget::MapEditor;
use PureMessage::Manager::Widget;
use PureMessage::Manager::Widget::ListEditor;
use PureMessage::MapFile;
use PureMessage::Map;
use PureMessage::Manager::Lang;
use PureMessage::ListManager;
use Data::Dumper;
use strict;

use base qw(PureMessage::Manager::Widget::ListEditor);

sub new {
        my $CLASS = shift;
        $CLASS = ref($CLASS) || $CLASS;
        my $this;
        eval {
           $this = $CLASS->SUPER::new(@_);
        };
        if ($@) {
                # then there was an error loading the list
                $this->{ERROR_LOADING_LIST} = $@;

                return PureMessage::Manager::Widget::Dialog->new( style => 'dlg_info',
                                                           msg => "$@", 
                                                           title=>text('list_init_died'),
                                                           buttons => [[ { type=>'image', text=>'Continue', icon=>'continue', href=>'index.cgi' } ]],
                                                         );           
 
        } 
        return $this;
        bless($this,$CLASS);
        return $this;
}

sub _init {
  my $this = shift;
  $this->config_default(name => ref($this),
                        width =>500,
                        css_class_1=>'medium',
                        css_class_2=>'light',
                        page_size => 11,
                        filter => '',
                        grid_columns=>1,
                        ddata => [{ description => 'Map From'},{description => 'Map To'}],
                        grid_columns => 2,
                        selectable => 1,
                       );

   $this->config_default(toolbar_width => $this->get('width'),
                         special => ["<input type=text size=30 name='" . $this->make_form_name('add_in1') . "'>",
                                    "<input type=text size=30 name='" . $this->make_form_name('add_in2') . "'>",
                                    ], 
                         filter_width => int($this->get('width') * 0.1)+1,
                         filter_table_width => $this->get('width')
                        );
                      


  my $rm = PureMessage::Resources::Manager->new();
  my $lr = $rm->find($this->get('list'));
  my $l;
  
  if($lr) {
      $this->{is_resource} = $rm->store_type();
      $l = $lr->native();
  } else {
     my $lm = PureMessage::ListManager->new(fetch => [qw(Map)]);
     $l = $lm->find_list($this->get('list'), writable => 1,edit_mode => 1);
  }
  
  $this->{LIST_INFO} = $l;
  $this->{LIST_TYPE} = 'map';
  if($l) {
  
          $this->{LIST} = PureMessage::rListWrapper->new($this->get('list'),$l);

  } else {
          die "Unable to create list object";
  }
    $this->config_default('grid_toolbar', sub {
                                              $this->widget(
                                                CLASS => 'PureMessage::Manager::Widget::ToolBar',
                                                embed =>1,
                                                name => $this->get('name'),
                                                buttons=>$this->_get_grid_toolbar_buttons(),
                                              )
                                              });


  die "Unable to Create List" unless $this->{LIST};
  $this->_prep_list();
}

sub _prep_list_item {
  my $this = shift;
  my $data = shift;
  my @result;
  @result = split(/: /,$data);
  return [$result[0],$result[1]];
}

# format web input. first array element is the map name, second is the value.
# Split on spaces to create a one to many map
sub _format_entry {
        my $this = shift;
        my $map = shift;
        use PureMessage::MapData::Entry;
        my @new_keys = map { s/\s//; $_ } split(' ',$map->[0]);
        my @new_value = map { s/\s//; $_ } split(' ',$map->[1]);

        my @temp = PureMessage::MapData::Entry->new(\@new_keys,\@new_value);
        return \@temp; 
}


sub _get_grid_toolbar_buttons {
        my $this = shift;
        my @buttons;
        my $q = $this->result_data();

        if (not $q->{'edit_selected'} or !($q->{'edit_selected'} and defined($q->{'selected_row'}))) {
          @buttons = ({ type=>'image',
                            icon=>'add',
                            name=>'add_new',
                            text=>'Add',
                         },
                         { type=>'image',
                            icon=>'delete',
                            text=>'Delete',
                            name=>'delete_selected',
                        });
        } else {
          @buttons = ({ type=>'image',
                       icon=>'cancel',
                       text=>'Cancel',
                       name=>'cancel',
                     });
        }
        return [\@buttons];
}


sub output {
        my $this = shift;
        my $result;
        my $data = $this->get('data');
        my $style = $this->get('style');
        my $result;
        if ($this->{ERROR_LOADING_LIST}) {
                $this->errors(text('list_init_died',$this->{ERROR_LOADING_LIST}));
                return $this->putErrors();
        }
        if ($this->{dirty}) {
            if ($this->{is_resource}) {
                $result .= $this->_info_box(text('list_is_resource',$this->{is_resource})) ;
                $result .= '<BR/>' . $this->_info_box(text('resource_list_is_dirty'));
            }
            else {
                $result .= $this->_info_box(text('list_is_dirty')); 
            }
        }

        $this->_prep_list($this->get('filter'));        
        my $w = new PureMessage::Manager::Widget;
        
        my $name = $this->get('name');

        my $width = $this->get('width');
        $result .= $this->putErrors();
        $result .= $this->_start_form();
        $result .= "<table cellpadding=0 cellspacing=0>";
        $result .= "<tr><td valign=top>";
        $result .= $this->_format_filter();
        $result .= $this->{GRID}->output;
        $result .= "</td></tr></table>\n";
        $result .= $this->format_state($this->get('state')) if $this->get('state');
        $result .= $this->_end_form();
        return $result;
}
    
    
1;

=pod

=cut


=head1 NAME

PureMessage::Manager::Widget::MapEditor

=head1 SYNOPSIS

 use PureMessage::Manger::Widget::MapEditor;

 my $WIDGET = new PureMessage::Manager::Widget();

 my $l = $WIDGET->widget(
                   CLASS =>'PureMessage::Manager::Widget::MapEditor',
                   name => 'edit_map',
                   list => $list_id,
                   page => 0,
                   action => $ENV{SCRIPT_NAME},
                   state => { id => $list_id},
                );

 $l->read_input();

 print $l->output();

=head1 DESCRIPTION

The MapEditor widget object can be used to generate html for a
C<PureMessage::Map> editor.

=head1 METHODS

=over 4

This widget shares the same interface, and behaves in the same manner
as the C<ListEdit> widget with the exeception that the C<list> constructor
parameter is expected to be the list_id of a valid C<PureMessage::Map>.

Please refer to that widgets documentation for details.


=back

=cut
