package PureMessage::Manager::Widget::PageBar;
use strict;

use Data::Dumper;
use PureMessage::Manager::Widget qw(buttons); # get the buttons func 

our @ISA = qw(PureMessage::Manager::Widget);

sub new {
        my $CLASS = shift;
        $CLASS = ref($CLASS) || $CLASS;
        my $this = $CLASS->SUPER::new(@_);

        bless($this,$CLASS);
        return $this;
}



sub output {
        my $this = shift;
        my $data;
        my $name = $this->get('name');
        my $current_page = $this->get('current_page');
        my $max_page = $this->get('max_page');
        my @buttons;
        
        $buttons[0] = $current_page > 0 ? { type=>'image',name=>'page_last',icon=>'previous'}
                                         : { type=>'custom', text=>'&nbsp;'};

        $buttons[1] = ($current_page >= 0 and $current_page < $max_page) ? { type=>'image',name=>'page_next',icon=>'next'}
                                                 : { type=>'custom', text=>'&nbsp;'};
        
        my ($s_page,$e_page);
        $s_page = ($current_page - 4 > 0) ? $current_page - 4 : 0;
        $e_page = ($current_page + 4 <= $max_page) ? $current_page +4 : $max_page;
        #return "<h1>page: $s_page -> $e_page ($current_page - $max_page)</h1>";
        
        my @pages;
        push(@pages,{ type=>'hidden',name=>'goto_page',
                      id=>$this->make_form_name('goto_page')
                    },
                    { type=>'hidden',name=>'current_page', value=> $current_page, 
                      id=>$this->make_form_name('current_page')
                    } 
            );
       
        
        if ($this->get('embed')) {
          foreach my $button(@buttons) {
            $button->{name} = $this->make_form_name($button->{name}) if $button->{name};
          }
          foreach my $page(@pages) {
            $page->{name} = $this->make_form_name($page->{name}) if $page->{name};
          }
        }
        
        for(my $i = $s_page; $i <= $e_page; $i++ ) {
                my $human_page = $i +1;
                if ($i == $current_page) {
                        push(@pages,{ type=>'text', text=>$human_page});
                } else {
                        push(@pages,{ type=>'text', text=>"$human_page", name=>$this->make_form_name('goto_page'),
                                      value=>$i, slave=>$this->make_form_name('goto_page')});
                }
        }
 
 
        my @bd = buttons(\@buttons,class => 'toolbar');
        my @pbd = buttons(\@pages,class=>'toolbar');
        my $item_count = $this->get('item_count');
        $max_page++; # make these into human counting numbers
        $current_page++;
        $data .= "<table cellpadding=0 cellspacing=0 width='100%' class=light>\n";
        $data .= "<tr>\n";
        $data .= "<td align=left>$bd[0]</td>";
        $data .= "<td align=center>";

        if (defined $item_count) {
          my $plural = ( $item_count == 0 or $item_count > 1 ) ? 's' : '';
          $data .= "<b>$item_count row$plural<BR>Viewing&nbsp;</b>";
        }

        $data .= "<b>Page $current_page of $max_page<BR>";
        
        if ($max_page > 1) {
                $data .= "Go to Page</b>";
                foreach my $page (@pbd) {
                        $data .= "&nbsp;" . $page;
                }
        }
        $data .= "</td>";
        $data .= "<td align=right>$bd[1]</td>";
        
        $data .= "</tr>\n";
        $data .= "</table>\n";

        return $data;
}




=pod

=head1 NAME

PureMessage::Manager::Widget::PageBar

=head1 SYNOPSIS

use PureMessage::Manger::Widget::PageBar;

my $WIDGET = new PureMessage::Manager::Widget();

my $pb = $WIDGET->widget(
                CLASS =>'PureMessage::Manager::Widget::PageBar',
                current_page=> 5,
                max_page => 8,
               );


print $pb->output();

=head1 DESCRIPTION

The PageBar widget object can be used to generate html for a
series of pages and the movement between them.

=head1 METHODS

=over 4

=item new( %options )

Create the object with the passed options. Options are:


=over 4

=item name

Name of this widget.

=item current_page

Page to display as current page.

=item max_page

Maximum page to show.

=item item_count

The count of items in the list. If this key is passed then a line will
be displayed on the generated pagebar indicating the count of items. 

=back


=item output()

Return the html that should be displayed for this widget.


=item read_input( $request )

This item will take no actions but the DATA variable for this
widget (accessible via C<get_result>) may contain the following
input:

=over 4

=item page_next

=item page_last

Will have a true value if the next/last button was clicked.

=item goto_page

Will contain the page to display if one of the page links was clicked.

=item current_page

Will contain the current_page that was displayed when this widget was
rendered.

=back

=back

=cut



1;


