package PureMessage::Manager::Widget::Template;
use PureMessage::Manager::Widget;
use HTML::Template;
use strict;

our @ISA = qw(PureMessage::Manager::Widget);

sub new {
        my $CLASS = shift;
        $CLASS = ref($CLASS) || $CLASS;
        my $this = $CLASS->SUPER::new(@_);
        $this->config_default('options'=>[],wrap=>0,width=>500,html_bgcolor=>'#F3F2EB');
        bless($this,$CLASS);
        return $this;
}

sub _found_element {
        my $this = shift;
        my $name = shift;
        my $data;
        if ($name eq 'TABLE_BEGIN') {
                my $width = $this->get('width');
                my $html_bgcolor = $this->get('html_bgcolor');
                my $bottom_width;
                $bottom_width = ($this->get('toolbar') or $this->get('suppress_bottom_border'))? 0 : 1;
                return "<table width='$width'  bgcolor=$html_bgcolor cellpadding=4 cellspacing=2 style='border-style: solid; border-color: #CDBF9E; border-width: 1px 1px ${bottom_width}px 1px'>";

        } elsif ($name eq 'TABLE_END') {
                return "</table>\n";
        } else {
                return "**UNKNOWN ELEMENT NAME**";
        }
}

sub _found_value {
        my $this = shift;
        my $name = shift;
        return $this->get_text($this->get($name));
}

sub _found_element_block {
        my $this = shift;
        my ($name, $data) = @_;
        if ($name eq 'FORM') {
                my $width = $this->get('width');
                my $html_bgcolor = $this->get('html_bgcolor');
                my $bottom_width;
                $bottom_width = ($this->get('toolbar') or $this->get('suppress_bottom_border'))? 0 : 1;
                return "<table width='$width'  bgcolor=$html_bgcolor cellpadding=4 cellspacing=2><tr><td>$data</td></tr></table>";
        } else {
                return "**UNKNOWN ELEMENT NAME**";
        
        }
}

# this is for the html template
sub _filter {
  my ($this,$line_ref) = @_;
  $$line_ref =~ s/<TMPL_VALUE\s+NAME="?([^\s">]+)"?(ESCAPE=.*)?>/$this->_found_value($1)/eig;
  $$line_ref =~ s|<TMPL_ELEMENT\s+NAME="?([^\s">]+)"?(?:ESCAPE=.*)?>(.*?)</TMPL_ELEMENT\s+NAME="?\1"?>|$this->_found_element_block($1,$2)|eigs;
  $$line_ref =~ s/<TMPL_ELEMENT\s+NAME="?([^\s">]+)"?(ESCAPE=.*)?>/$this->_found_element($1)/eig;
  #" to make vim behave
}

sub output {
        my $this = shift;
        my $data;
        my %options = @{$this->get('options')};
        my $data;
        if ($options{filter}) {
          $options{filter} = [{ 'sub'=>sub { $this->_filter(@_)},format=>'scalar' },
                              { format=>'scalar','sub'=>$options{filter}}
                             ];
        } else {
          $options{filter} = sub { $this->_filter(@_) };
        }
        my $template = template($this->get('filename'),%options,); 
        $template->param($this->get('data'));

        if($this->get('wrap')) {
                my $width = $this->get('width');
                my $html_bgcolor = $this->get('html_bgcolor');
                my $bottom_width;
                $bottom_width = ($this->get('toolbar') or $this->get('suppress_bottom_border'))? 0 : 1;
                $data .= "<table width='$width'  bgcolor=$html_bgcolor border=0 cellpadding=4 cellspacing=2 style='border-style: solid; border-color: #CDBF9E; border-width: 1px 1px ${bottom_width}px 1px'>";
                if (my $title = $this->get('title')) {
                        $data .= "<tr><td><b>$title</b></td></tr>\n";
                }
                $data .= "<tr><td>\n";
                $data .= $template->output();
                $data .= "</td></tr></table>\n";
                if (my $tb = $this->get('toolbar')) {
                        # take over this toolbar
                        $tb->{name} = $this->get('name');
                        $tb->{embed} =1;
                        $tb->{width} = $this->get('width');
                        #$data .= $tb->output();
                }

        } else {
                $data .= $template->output();
        }
        return $data;
}
    
1;



