package PureMessage::Manager::Widget::ToolBar;
use strict;
use Data::Dumper;
use PureMessage::Manager::Widget qw(buttons); # get the buttons func 

our @ISA = qw(PureMessage::Manager::Widget);

sub new {
        my $CLASS = shift;
        $CLASS = ref($CLASS) || $CLASS;
        my $this = $CLASS->SUPER::new(@_);

        bless($this,$CLASS);
        return $this;
}

sub output {
        my $this = shift;
        my $data = '';
        my $header = '';
        my $footer = '';
        my $buttons = $this->get('buttons');
        my $name = $this->get('name');
        if ($this->get('embed')) {
          foreach my $button(@{$buttons->[0]}) {
            $button->{name} = $this->make_form_name($button->{name}) if $button->{name};
          }
        }
        
        # add our state hash if it's there
        if(my $state = $this->get('state')) {
                push(@{$buttons->[0]},$this->state_buttons($state));
        }
        
        my @bd = buttons(@{$buttons},class => "tableButton", style=>"toolbar");
        for(my $i = 0; $i <= $#bd; $i++) {
                if($buttons->[0]->[$i]->{type} ne 'hidden') {
                    $data .= <<EOT;
            <td class="tableContent">
                <table border="0" cellpadding="1" cellspacing="0" width="100%">
                    <tr>
                        <td class="tableButtonOutline">
                        $bd[$i]
                        </td>
                    </tr>
                </table>
            </td>
EOT
                } else {
                        # don't wrap hidden fields in td's
                        $header .= "$bd[$i]";
                }
        }

        # what kind of wrapping?
        my $temp;
        
        if ($temp = $this->get('action')) {
                # then we have to wrap this table in a form 
                $header = "\n<form action='$temp' method=post style='margin: 0'>\n" . $header;
                $footer .= "\n</form>\n";
        }
        # TODO prune out table and width param handling stuff, subsequent
        # changes have invalidated it and it no longer works.

        # do we have column formatting data or not?
        my $table_format = $this->get('table');
        
        my $skip_wrap = $table_format ? 1 : 0; # so we know where the data came from
       
        $table_format = [[1,1],['10%','90%']] unless $table_format;
       
        my $width = $this->get('width');
        $width = 600 unless $width;

        $data = <<EOF;
$header
<table border="0" cellpadding="1" cellspacing="0">
<tr>
$data
</tr>
</table>
$footer
EOF

        return $data;
}


1;

=pod

=cut


=head1 NAME

PureMessage::Manager::Widget::ToolBar

=head1 SYNOPSIS

 use PureMessage::Manger::Widget::ToolBar;

 my $WIDGET = new PureMessage::Manager::Widget();

 my @buttons = ( { type=>text, href=>"http://www.sophos.com", text=>"Sophos" }
                 { type=>image, icon=>"checkmark", name=>"doit", value=>"yes" }
               );

 my %options = ( target => 'new' );

 my $tb = $WIDGET->widget(
                 CLASS =>'PureMessage::Manager::Widget::ToolBar',
                 buttons => [\@buttons,%options],
                 embed => 1,
                 action => 'index.cgi',
                );

 # Toolbar wrapped in it's own form with submits sent to
 # index.cgi. One button recognized only by this toolbar.
 # the other opens a new window to http://www.sophos.com

print $tb->output();

=head1 DESCRIPTION

The ToolBar widget object can be used to generate html for a
button toolbar. 

=head1 METHODS

=over 4

=item new( %options)

Creates an toolbar object. Options are:

=over 4

=item buttons

The buttons to dislay. This should be an array reference containing data 
matching input expected by the C<buttons> function of Widget.
This data will be passed directly to the buttons function and used as the
action buttons on the generated toolbar.

 my @buttons = ( { type=>text, text=>'hello' }, ... );

 my $tb = new PureMessage::Manager::Widget::Toolbar ( 
                buttons => [\@buttons,target => 'new' ],
              );

=item embed

Modify the name parameters of any buttons using the C<make_form_name> 
function so that input is recognized only by this widget.

=item state

A hash reference that contains state key/value information that should
be encoded as hidden buttons and submitted with any form.

=item action

If this key is present then the generated html is wrapped in it's own
form tags with it's action set to the value of this key. The form
method is GET.

=back

=item output()

Returns the html that can be used to display this Widget.

=item read_input( $request )

This item will take no actions but the DATA variable for this
widget (accessible via C<get_result>) will contain any button
data destined for this widget.

=cut



