package PureMessage::Service::http;
use strict;
use PureMessage::Manager::Service qw(
        manager_start
        manager_stop
        manager_restart
        manager_status
        manager_is_running
);
use PureMessage::Manager::Config qw(%web_config);
use Socket;

use base 'PureMessage::Service';

if ( $@ ) { die "Error: $@ $!\n" }

sub Available {
    my $class = shift;
    my @avail;
    push(@avail, $class->new(name => "Manager"));
    return @avail;
}

sub _bg {
    my $pid = fork();
    die "Can't fork: $!" unless defined $pid;
    return $pid if $pid;

    open(STDIN, "</dev/null");
    open(STDOUT, ">/dev/null");
    open(STDERR, ">/dev/null");
    return 0;
}

sub start {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
           return $pid;
        }
    }
    manager_start();
    exit if $opts{bg};
}

sub stop {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
            return $pid;
        }
    }
    manager_stop();
    exit if $opts{bg};
}

sub restart {
    my($self, %opts) = @_;
    if ($opts{bg}) {
        if (my $pid = _bg()) {
            return $pid;
        }
    }
    manager_restart();
    exit if $opts{bg};
}

sub status {
    my $self = shift;
    my $info = manager_status();
    $info->{ready} = $self->running();
    $info->{summary} = "PureMessage Manager";
    $info->{summary} .= ", port " . $info->{'port'};
    $info->{summary} .= " SSL mode"
        if $info->{https};
    $info->{running} = $self->running();
    $info;
}

sub running {
    my $self = shift;
    return manager_is_running();
}

# restart_miniserv()
# Send a HUP signal to miniserv
sub soft_restart {
    local $SIG{TERM} = 'IGNORE'; 
    my $self = shift;
    my ($pid, $addr, $i);
    my $wait = 20;
    $web_config{'inetd'} && return;
    open(PID, $web_config{'pidfile'}) || return "Failed to open pid file";
    chop($pid = <PID>);
    close(PID);
    if (!$pid) { return "Invalid pid file"; }

    if(my $code = shift) {
        if(open(PID, ">$web_config{pidfile}")) {
            print PID "$pid\n$code\n";
            close(PID);
            $wait = 600; # give it longer if it has something to do
        } else {
            return "Failed to write hook to PID file: $!";
        }
    }

    kill ('HUP', $pid); # XXX log this
    
    # wait for miniserv to come back up
    $addr = inet_aton($web_config{'bind'} ? $web_config{'bind'} : "127.0.0.1");
    my $ok = 0;
    no strict;
    for(my $i=0; $i<$wait; $i++) {
	sleep(1);
	socket(STEST, PF_INET, SOCK_STREAM, getprotobyname("tcp"));
	my $rv = connect(STEST, sockaddr_in($web_config{'port'}, $addr));
	close(STEST);
	last if ($rv && ++$ok >= 2);
    }
    if ($i == $wait) { return "Failed to restart Webmin server!"; }
    return 0;
}

1;
